@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.12 98/02/16 15:55:38 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.13 98/03/06 07:22:47 abe Exp $";
 #endif
 
 
@@ -60,8 +60,8 @@ ckfd_range(first, dash, last, lo, hi)
  * See if the range character pointers make sense.
  */
 	if (first >= dash || dash >= last) {
-	    (void) fprintf(stderr, "%s: illegal FD range for -d: %s\n",
-		Pn, first);
+	    (void) fprintf(stderr, "%s: illegal FD range for -d: ", Pn);
+	    safestrprt(first, stderr, 1);
 	    return(1);
 	}
 /*
@@ -72,8 +72,8 @@ ckfd_range(first, dash, last, lo, hi)
 
 FD_range_nondigit:
 
-		(void) fprintf(stderr, "%s: non-digit in -d FD range: %s\n",
-		    Pn, first);
+		(void) fprintf(stderr, "%s: non-digit in -d FD range: ", Pn);
+		safestrprt(first, stderr, 1);
 		return(1);
 	    }
 	    *lo = (*lo * 10) + (int)(*cp - '0');
@@ -84,8 +84,8 @@ FD_range_nondigit:
 	    *hi = (*hi * 10) + (int)(*cp - '0');
 	}
 	if (*lo >= *hi) {
-	    (void) fprintf(stderr, "%s: -d FD range's low >= its high: %s\n",
-		Pn, first);
+	    (void) fprintf(stderr, "%s: -d FD range's low >= its high: ", Pn);
+	    safestrprt(first, stderr, 1);
 	    return(1);
 	}
 	return(0);
@@ -104,6 +104,7 @@ ck_file_arg(i, ac, av)
 {
 	unsigned char ad, an;
 	short err = 0;
+	int errno_save;
 	char *fnm, *fsnm, *path;
 	int fsm, ftype, j;
 	MALLOC_S l;
@@ -133,11 +134,13 @@ ck_file_arg(i, ac, av)
 		continue;
 	    }
 	/*
-	 * Remove extra terminating `/'.
+	 * Remove a terminating `/' from Readlink()-expanded path, but never
+	 * from an argument-specified path.  Don't remove `/' from a single
+	 * character path name, either.
 	 *
 	 * Check for file system argument.
 	 */
-	    if ((j = strlen(path)) > 1 && path[j-1] == '/')
+	    if (path != av[i] && (j = strlen(path)) > 1 && path[j-1] == '/')
 		path[j-1] = '\0';
 	    for (ftype = 1, mp = Mtab, nm = 0;
 		 (Ffilesys != 1) && mp;
@@ -185,8 +188,8 @@ ck_file_arg(i, ac, av)
 		mmp[nm++] = mp;
 	    }
 	    if (Ffilesys == 2 && nm == 0) {
-		(void) fprintf(stderr, "%s: not a file system: %s\n",
-		    Pn, av[i]);
+		(void) fprintf(stderr, "%s: not a file system: ", Pn);
+		safestrprt(av[i], stderr, 1);
 		err = 1;
 		continue;
 	    }
@@ -219,8 +222,10 @@ ck_file_arg(i, ac, av)
 		 * Stat the path to obtain its characteristics.
 		 */
 		    if (statsafely(fnm, &sb) != 0) {
-			(void) fprintf(stderr, "%s: status error on %s: %s\n",
-			    Pn, fnm, strerror(errno));
+			errno_save = errno;
+			(void) fprintf(stderr, "%s: status error on ", Pn);
+			safestrprt(fnm, stderr, 0);
+			(void) fprintf(stderr, ": %s\n", strerror(errno_save));
 			err = 1;
 			break;
 		    }
@@ -298,34 +303,32 @@ ck_file_arg(i, ac, av)
 		    sfp->name = fnm;
 		    an = 0;
 		} else {
-		    if (!(sfp->name=(char *)malloc((MALLOC_S)(strlen(fnm)+1))))
-		    {
+		    if (!(sfp->name = mkstrcpy(fnm, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr,
-			    "%s: no space for file name %s\n", Pn, fnm);
+			    "%s: no space for file name: ", Pn);
+			safestrprt(fnm, stderr, 1);
 			Exit(1);
 		    }
-		    (void) strcpy(sfp->name, fnm);
 		    an = 1;
 		}
 		if (!fsnm || fsnm == path) {
 		    sfp->devnm = fsnm;
 		    ad = 0;
 		} else {
-		    if (!(sfp->devnm=(char *)malloc((MALLOC_S)(strlen(fsnm)+1)))
-		    ) {
+		    if (!(sfp->devnm = mkstrcpy(fsnm, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr,
-			    "%s: no space for file system name %s\n", Pn, fsnm);
+			    "%s: no space for file system name: ", Pn);
+			safestrprt(fsnm, stderr, 1);
 			Exit(1);
 		    }
-		    (void) strcpy(sfp->devnm, fsnm);
 		    ad = 1;
 		}
-		if (!(sfp->aname=(char *)malloc((MALLOC_S)(strlen(av[i])+1)))) {
+		if (!(sfp->aname = mkstrcpy(av[i], (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr,
-			"%s: no space for argument file name %s\n", Pn, av[i]);
+			"%s: no space for argument file name: ", Pn);
+			safestrprt(av[i], stderr, 1);
 		    Exit(1);
 		}
-		(void) strcpy(sfp->aname, av[i]);
 
 #if	defined(HASPROCFS)
 	    /*
@@ -373,8 +376,9 @@ ck_file_arg(i, ac, av)
 		if (!(pfi = (struct procfsid *)malloc((MALLOC_S)
 			     sizeof(struct procfsid))))
 		{
-		    (void) fprintf(stderr, "%s: no space for %s ID: %s\n",
-			Pn, Mtprocfs->dir, path);
+		    (void) fprintf(stderr, "%s: no space for %s ID: ",
+			Pn, Mtprocfs->dir);
+		    safestrprt(path, stderr, 1);
 		    Exit(1);
 		}
 		pfi->pid = pid;
@@ -416,84 +420,83 @@ ctrl_dcache(c)
 	MALLOC_S len;
 	int rc = 0;
 	
-	if (c == NULL) {
-		(void) fprintf(stderr,
-			"%s: no device cache option control string\n", Pn);
-		return(1);
+	if (!c) {
+	    (void) fprintf(stderr,
+		"%s: no device cache option control string\n", Pn);
+	    return(1);
 	}
 /*
  * Decode argument function character.
  */
 	switch (*c) {
 	case '?':
-		if (*(c+1) != '\0') {
-			(void) fprintf(stderr,
-				"%s: nothing should follow -D?\n", Pn);
-			return(1);
-		}
-		DChelp = 1;
-		return(0);
+	    if (*(c+1) != '\0') {
+		(void) fprintf(stderr, "%s: nothing should follow -D?\n", Pn);
+		return(1);
+	    }
+	    DChelp = 1;
+	    return(0);
 	case 'b':
 	case 'B':
-		if (Setuidroot
+	    if (Setuidroot
 
 #if	!defined(WILLDROPGID)
-		||  Myuid
+	    ||  Myuid
 #endif	/* !defined(WILLDROPGID) */
 
-		)
-			rc = 1;
-		else
-			DCstate = 1;
-		break;
+	    )
+		rc = 1;
+	    else
+		DCstate = 1;
+	    break;
 	case 'r':
 	case 'R':
-		if (Setuidroot && *(c+1))
-			rc = 1;
-		else
-			DCstate = 2;
-		break;
+	    if (Setuidroot && *(c+1))
+		rc = 1;
+	    else
+		DCstate = 2;
+	    break;
 	case 'u':
 	case 'U':
-		if (Setuidroot
+	    if (Setuidroot
 
 #if	!defined(WILLDROPGID)
-		||  Myuid
+	    ||  Myuid
 #endif	/* !defined(WILLDROPGID) */
 
-		)
-			rc = 1;
-		else
-			DCstate = 3;
-		break;
+	    )
+		rc = 1;
+	    else
+		DCstate = 3;
+	    break;
 	case 'i':
 	case 'I':
-		if (*(c+1) == '\0') {
-			DCstate = 0;
-			return(0);
-		}
-		/* fall through */
+	    if (*(c+1) == '\0') {
+		DCstate = 0;
+		return(0);
+	    }
+	    /* fall through */
 	default:
-		(void) fprintf(stderr, "%s: unknown -D option: %s\n", Pn, c);
-		return(1);
+	    (void) fprintf(stderr, "%s: unknown -D option: ", Pn);
+	    safestrprt(c, stderr, 1);
+	    return(1);
 	}
 	if (rc) {
-		(void) fprintf(stderr,
-			"%s: -D option restricted to root: %s\n", Pn, c);
-		return(1);
+	    (void) fprintf(stderr, "%s: -D option restricted to root: ", Pn);
+	    safestrprt(c, stderr, 1);
+	    return(1);
 	}
 /*
  * Skip to optional path name and save it.
  */
 	for (c++; *c && (*c == ' ' || *c == '\t'); c++)
-		;
-	if ((len = strlen(c))) {
-		if ((DCpathArg = (char *)malloc(len + 1)) == NULL) {
-			(void) fprintf(stderr,
-				"%s: no space for -D path: %s\n", Pn, c);
-			Exit(1);
-		}
-		(void) strcpy(DCpathArg, c);
+	    ;
+	if (strlen(c)) {
+	    if (!(DCpathArg = mkstrcpy(c, (MALLOC_S)NULL))) {
+		(void) fprintf(stderr, "%s: no space for -D path: ", Pn);
+		safestrprt(c, stderr, 1);
+		Exit(1);
+	    }
 	}
 	return(0);
 }
@@ -515,16 +518,15 @@ enter_fd(f)
 /*
  *  Check for non-empty list and make a copy.
  */
-	if (f == (char *)NULL || (len = strlen(f) + 1) < 2) {
+	if (!f || (len = strlen(f) + 1) < 2) {
 	    (void) fprintf(stderr, "%s: no file descriptor specified\n", Pn);
 	    return(1);
 	}
-	if ((fc = (char *)malloc(len)) == (char *)NULL) {
-	    (void) fprintf(stderr, "%s: no space for %d byte fd string: %s\n",
-		Pn, (int)len, f);
+	if (!(fc = mkstrcpy(f, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no space for fd string: ", Pn);
+	    safestrprt(f, stderr, 1);
 	    Exit(1);
 	}
-	(void) strcpy(fc, f);
 /*
  * Isolate each file descriptor in the comma-separated list, then enter it
  * in the file descriptor string list.  If a descriptor has the form:
@@ -581,7 +583,7 @@ enter_id(ty, p)
 	int i, id, mx, n;
 	struct int_lst *s;
 
-	if (p == NULL) {
+	if (!p) {
 	    (void) fprintf(stderr, "%s: no process%s ID specified\n",
 		Pn, (ty == PGRP) ? " group" : "");
 	    return(1);
@@ -601,8 +603,9 @@ enter_id(ty, p)
 	    s = Spid;
 	    break;
 	default:
-	    (void) fprintf(stderr, "%s: enter_id \"%s\", invalid type: %d\n",
-		Pn, p, ty);
+	    (void) fprintf(stderr, "%s: enter_id \"", Pn);
+	    safestrprt(p, stderr, 0);
+	    (void) fprintf(stderr, "\", invalid type: %d\n", ty);
 	    Exit(1);
 	}
 /*
@@ -622,8 +625,9 @@ enter_id(ty, p)
 #endif	/* __STDC__ */
 
 		{
-		    (void) fprintf(stderr, "%s: illegal process%s ID: %s\n",
-			Pn, (ty == PGRP) ? " group" : "", p);
+		    (void) fprintf(stderr, "%s: illegal process%s ID: ",
+			Pn, (ty == PGRP) ? " group" : "");
+		    safestrprt(p, stderr, 1);
 		    return(1);
 		}
 		id = (id * 10) + *cp - '0';
@@ -685,8 +689,8 @@ enter_network_address(na)
 {
 	unsigned char *ap;
 	int ep, pr, sp;
-	struct hostent *he = (struct hostent *) NULL;
-	char *hn = NULL;
+	struct hostent *he = (struct hostent *)NULL;
+	char *hn = (char *)NULL;
 	MALLOC_S l;
 	struct nwad n;
 	char *p, *wa;
@@ -696,14 +700,14 @@ enter_network_address(na)
 	char *sn = (char *)NULL;
 	MALLOC_S snl = 0;
 
-	if (na == NULL) {
+	if (!na) {
 	    (void) fprintf(stderr, "%s: no network address specified\n", Pn);
 	    return(1);
 	}
 /*
  * Initialize network address structure.
  */
-	n.proto = NULL;
+	n.proto = (char *)NULL;
 	n.a[0] = n.a[1] = n.a[2] = n.a[3] = 0;
 /*
  * Process protocol name, optionally followed by a '@' and a host name or
@@ -714,10 +718,12 @@ enter_network_address(na)
 	    for (p = wa; *wa && *wa != '@' && *wa != ':'; wa++)
 		;
 	    if ((l = wa - p)) {
-		if ((n.proto = (char *)malloc(l + 1)) == NULL) {
+		if (!(n.proto = mkstrcat(p, l, (char *)NULL, -1, (char *)NULL,
+			        -1, (MALLOC_S *)NULL)))
+		{
 		    (void) fprintf(stderr,
-			"%s: no space for protocol name from %s\n",
-			Pn, na);
+			"%s: no space for protocol name from ", Pn);
+		    safestrprt(na, stderr, 1);
 nwad_exit:
 		    if (n.proto)
 			(void) free((FREE_P *)n.proto);
@@ -727,18 +733,16 @@ nwad_exit:
 			(void) free((FREE_P *)sn);
 		    return(1);
 		}
-		(void) strncpy(n.proto, p, l);
-		n.proto[l] = '\0';
 	    /*
 	     * The protocol name should be "tcp" or "udp".
 	     */
 		if (strcasecmp(n.proto, "tcp") != 0
 		&&  strcasecmp(n.proto, "udp") != 0)
 		{
-			(void) fprintf(stderr,
-		    	    "%s: unknown protocol name (%s) in %s\n",
-			    Pn, n.proto, na);
-			goto nwad_exit;
+		    (void) fprintf(stderr,
+			"%s: unknown protocol name (%s) in ", Pn, n.proto);
+		    safestrprt(na, stderr, 1);
+		    goto nwad_exit;
 		}
 	    /*
 	     * Convert protocol name to lower case.
@@ -755,11 +759,12 @@ nwad_exit:
  */
 	if (*wa == '@') {
 	    wa++;
-	    if ( ! *wa || *wa == ':') {
+	    if (!*wa || *wa == ':') {
 
 unacc_address:
 		(void) fprintf(stderr,
-		    "%s: unacceptable Internet address in %s\n", Pn, na);
+		    "%s: unacceptable Internet address in ", Pn);
+		safestrprt(na, stderr, 1);
 		goto nwad_exit;
 	    }
 	    if (*wa < '0' || *wa > '9') {
@@ -769,16 +774,18 @@ unacc_address:
 		for (p = wa; *p && *p != ':'; p++)
 		    ;
 		if ((l = p - wa)) {
-		    if ((hn = (char *)malloc(l + 1)) == NULL) {
-			fprintf(stderr,
-			    "%s: no space for host name from %s\n", Pn, na);
+		    if (!(hn = mkstrcat(wa, l, (char *)NULL, -1, (char *)NULL,
+			       -1, (MALLOC_S *)NULL)))
+		    {
+			(void) fprintf(stderr,
+			    "%s: no space for host name: ", Pn);
+			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
-		    (void) strncpy(hn, wa, l);
-		    hn[l] = '\0';
 		    if ((he = gethostbyname(hn)) == NULL) {
-			fprintf(stderr, "%s: unknown host name (%s) in %s\n",
-			    Pn, hn, na);
+			fprintf(stderr, "%s: unknown host name (%s) in ",
+			    Pn, hn);
+			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
 		    ap = (unsigned char *)he->h_addr;
@@ -835,7 +842,8 @@ unacc_address:
 
 unacc_port:
 	    (void) fprintf(stderr,
-		"%s: unacceptable port specification in %s\n", Pn, na);
+		"%s: unacceptable port specification in ", Pn);
+	    safestrprt(na, stderr, 1);
 	    goto nwad_exit;
 	}
 	for (++wa; wa && *wa; wa++) {
@@ -849,9 +857,10 @@ unacc_port:
 		 */
 		    for (p = wa; *wa && *wa != ','; wa++)
 			;
-		    if ((l = wa - p) == 0) {
+		    if (!(l = wa - p)) {
 			(void) fprintf(stderr,
-			    "%s: invalid service name: %s\n", Pn, na);
+			    "%s: invalid service name: ", Pn);
+			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
 		    if (sn) {
@@ -865,7 +874,8 @@ unacc_port:
 		    }
 		    if (!sn) {
 			(void) fprintf(stderr,
-			    "%s: no space for service name: %s\n", Pn, na);
+			    "%s: no space for service name: ", Pn);
+			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
 		    (void) strncpy(sn, p, l);
@@ -878,8 +888,9 @@ unacc_port:
 		     */
 			if (!(se = getservbyname(sn, n.proto))) {
 			    (void) fprintf(stderr,
-				"%s: unknown service %s for %s in %s\n",
-				Pn, sn, n.proto, na);
+				"%s: unknown service %s for %s in ",
+				Pn, sn, n.proto);
+			    safestrprt(na, stderr, 1);
 			    goto nwad_exit;
 			}
 			pt = (int)ntohs(se->s_port);
@@ -895,7 +906,8 @@ unacc_port:
 			    pu = (int)ntohs(se1->s_port);
 			if (!se && !se1) {
 			    (void) fprintf(stderr,
-				"%s: unknown service %s in %s\n", Pn, sn, na);
+				"%s: unknown service %s in ", Pn, sn);
+			    safestrprt(na, stderr, 1);
 			    goto nwad_exit;
 			}
 			if (se && se1 && pt != pu) {
@@ -982,14 +994,13 @@ enter_nwad(n, sp, ep, s, he)
 /*
  * Allocate space for the argument specification.
  */
-	if ((ac = strlen(s))) {
-	    if (!(n->arg = (char *)malloc(ac + 1))) {
+	if (strlen(s)) {
+	    if (!(n->arg = mkstrcpy(s, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr,
-		    "%s: no space for Internet argument: %s\n",
-		    Pn, s);
+		    "%s: no space for Internet argument: ", Pn);
+		safestrprt(s, stderr, 1);
 		Exit(1);
 	    }
-	    (void) strcpy(n->arg, s);
 	} else
 	    n->arg = (char *)NULL;
 /*
@@ -1006,8 +1017,8 @@ enter_nwad(n, sp, ep, s, he)
 	    &&  nc.a[0] == 0 && nc.a[1] == 0 && nc.a[2] == 0 && nc.a[3] == 0
 	    &&  sp == -1) {
 		(void) fprintf(stderr,
-		    "%s: incomplete Internet address specification:  %s\n",
-		    Pn, s);
+		    "%s: incomplete Internet address specification: ", Pn);
+		safestrprt(s, stderr, 1);
 		return(1);
 	    }
 	/*
@@ -1016,8 +1027,9 @@ enter_nwad(n, sp, ep, s, he)
 	 */
 	    if (na >= MAXNWAD) {
 		(void) fprintf(stderr,
-		    "%s: network address limit (%d) exceeded: %s\n",
-		    Pn, MAXNWAD, s);
+		    "%s: network address limit (%d) exceeded: ",
+		    Pn, MAXNWAD);
+		safestrprt(s, stderr, 1);
 		return(1);
 	    }
 	/*
@@ -1025,7 +1037,8 @@ enter_nwad(n, sp, ep, s, he)
 	 */
 	    if ((np = (struct nwad *)malloc(sizeof(struct nwad))) == NULL) {
 		(void) fprintf(stderr,
-		    "%s: no space for network address from: %s\n", Pn, s);
+		    "%s: no space for network address from: ", Pn);
+		safestrprt(s, stderr, 1);
 		return(1);
 	    }
 	/*
@@ -1069,22 +1082,23 @@ enter_str_lst(opt, s, lp)
 	MALLOC_S len;
 	struct str_lst *lpt;
 
-	if (s == NULL || *s == '-') {
-		(void) fprintf(stderr, "%s: missing %s option value\n",
-			Pn, opt);
-		return(1);
+	if (!s || *s == '-') {
+	    (void) fprintf(stderr, "%s: missing %s option value\n",
+		Pn, opt);
+	    return(1);
 	}
-	len = strlen(s);
-	if ((cp = (char *)malloc(len + 1)) == NULL) {
-		(void) fprintf(stderr, "%s: no string copy space: %s\n",
-			Pn, s);
-		return(1);
+	if (!(cp = mkstrcpy(s, &len))) {
+	    (void) fprintf(stderr, "%s: no string copy space: ", Pn);
+	    safestrprt(s, stderr, 1);
+	    return(1);
 	}
 	if ((lpt = (struct str_lst *)malloc(sizeof(struct str_lst))) == NULL) {
-		(void) fprintf(stderr, "%s: no list space: %s\n", Pn, s);
-		return(1);
+	    (void) fprintf(stderr, "%s: no list space: ", Pn);
+	    safestrprt(s, stderr, 1);
+	    (void) free((FREE_P *)cp);
+	    return(1);
 	}
-	(void) strcpy(cp, s);
+	lpt->f = 0;
 	lpt->str = cp;
 	lpt->len = (int)len;
 	lpt->next = *lp;
@@ -1108,7 +1122,7 @@ enter_uid(u)
 	char *s;
 	uid_t uid;
 
-	if (u == NULL) {
+	if (!u) {
 	    (void) fprintf(stderr, "%s: no UIDs specified\n", Pn);
 	    return(1);
 	}
@@ -1120,7 +1134,8 @@ enter_uid(u)
 	    for (excl = i = j = lnml = nn = uid = 0; *s && *s != ','; i++, s++)
 	    {
 		if (j >= LOGINML-1) {
-		    (void) fprintf(stderr, "%s: illegal UID in %s\n", Pn, u);
+		    (void) fprintf(stderr, "%s: illegal UID in ", Pn);
+		    safestrprt(u, stderr, 1);
 		    return(1);
 		}
 		if (i == 0 && *s == '^') {
@@ -1146,8 +1161,8 @@ enter_uid(u)
 	    if (nn) {
 	       lnm[lnml++] = '\0';
 		if ((pw = getpwnam(lnm)) == NULL) {
-		    (void) fprintf(stderr, "%s: can't get UID for %s\n",
-			Pn, lnm);
+		    (void) fprintf(stderr, "%s: can't get UID for ", Pn);
+		    safestrprt(lnm, stderr, 1);
 		    err = 1;
 		    continue;
 		} else
@@ -1203,12 +1218,11 @@ enter_uid(u)
 		}
 	    }
 	    if (nn) {
-		if (!(lp = (char *)malloc(lnml))) {
-		    (void) fprintf(stderr, "%s: no space for login: %s\n",
-			Pn, lnm);
+		if (!(lp = mkstrcpy(lnm, (MALLOC_S)NULL))) {
+		    (void) fprintf(stderr, "%s: no space for login: ", Pn);
+		    safestrprt(lnm, stderr, 1);
 		    Exit(1);
 		}
-		(void) strcpy(lp, lnm);
 		Suid[Nuid].lnm = lp;
 	    } else
 		Suid[Nuid].lnm = (char *)NULL;
