@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.7 98/02/13 12:42:49 abe Exp $
+ * $Id: dlsof.h,v 1.8 98/03/06 08:04:29 abe Exp $
  */
 
 
@@ -293,7 +293,7 @@ extern int ClonePtc;
  */
 
 struct l_vfs {
-	struct vfs *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
@@ -355,7 +355,7 @@ extern struct nlist AFSnl[];	/* AFS kernel symbol name list table */
 extern char *AFSApath;		/* alternate AFS name list path (from -a) */
 #  endif	/* defined(HASAOPT) */
 
-extern struct vfs *AFSVfsp;	/* AFS struct vfs kernel pointer */
+KA_T AFSVfsp;			/* AFS struct vfs kernel pointer */
 # endif	/* defined(HAS_AFS) */
 
 extern int Kd;
