@@ -31,13 +31,18 @@
 
 
 /*
- * $Id: dlsof.h,v 1.5 98/02/13 12:43:27 abe Exp $
+ * $Id: dlsof.h,v 1.6 98/03/06 08:21:23 abe Exp $
  */
 
 
 #if	!defined(HPUX_LSOF_H)
 #define	HPUX_LSOF_H	1
 
+# if	HPUXV>=1030
+#include <sys/types.h>
+#include <fcntl.h>
+# endif	/* HPUXV>=1030 */
+
 #include <stdlib.h>
 #include <dirent.h>
 #include <mntent.h>
@@ -50,18 +55,42 @@
 #include <sys/vnode.h>
 # endif	/* HPUXV<1020 */
 
+# if	HPUXV>=1030
+/*
+ * Include header files for HP-UX 10.30 and up that have been
+ * manufactured with q4 and hand editing.
+ */
+
+#include "lla.h"
+#include "proc.h"
+#include "rnode.h"
+#include "nfs_clnt.h"
+#include "vnode.h"
+# endif	/* HPUXV>=1030 */
+
 #include <sys/domain.h>
 
 # if	HPUXV>=1020
-#define	_INCLUDE_STRUCT_FILE	1
+#define	_INCLUDE_STRUCT_FILE
 # endif	/* HPUXV>=1020 */
 
+# if	HPUXV>=1030
+struct uio {		/* to satisfy function prototypes in <sys/file.h> */
+	int dummy;
+};
+# endif	/* HPUXV>=1030 */
+
 #include <sys/file.h>
 
 # if	HPUXV>=1020
 #undef	_INCLUDE_STRUCT_FILE
 # endif	/* HPUXV>=1020 */
 
+# if	HPUXV>=1030
+#include <sys/stream.h>
+#include "sth.h"
+# endif	/* HPUXV>=1030 */
+
 #include <sys/mbuf.h>
 
 # if	HPUXV>=800
@@ -80,18 +109,45 @@
 #include <sys/socketvar.h>
 #include <netinet/in.h>
 #include <net/route.h>
+
+# if	HPUXV<1030
 #include <net/raw_cb.h>
 #include <netinet/in_pcb.h>
+# endif	/* HPUXV<1030 */
+
 #include <netinet/ip_var.h>
 #include <netinet/tcp.h>
 #include <netinet/tcpip.h>
+
+# if	HPUXV<1030
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
+# else	/* HPUXV>=1030 */
+#include <sys/tihdr.h>
+/*
+ * Include header files for HP-UX 10.30 and up that have been
+ * manufactured with q4 and hand editing.
+ */
+
+#include "ipc_s.h"
+#include "tcp_s.h"
+#include "udp_s.h"
+# endif	/* HPUXV<1030 */
+
+# if	HPUXV>=1030
+#undef	TCP_NODELAY
+#undef	TCP_MAXSEG
+# endif	/* HPUXV>=1030 */
+
 #include <rpc/types.h>
 #include <rpc/rpc.h>
 #include <rpc/pmap_prot.h>
 
+# if	HPUXV>=1030
+#include <rpc/clnt_soc.h>
+# endif	/* HPUXV>=1030 */
+
 # if	HPUXV>=1000
 #include <rpc/xdr.h>
 #include <rpc/auth.h>
@@ -102,8 +158,12 @@
 # endif	/* HPUXV>=1000 */
 
 #include <nfs/nfs.h>
+
+# if	HPUXV<1030
 #include <nfs/nfs_clnt.h>
 #include <nfs/rnode.h>
+# endif	/* HPUXV<1030 */
+
 #include <nfs/snode.h>
 
 # if	HPUXV>=1000
@@ -112,11 +172,11 @@
 #undef	_KERNEL
 # endif	/* HPUXV>=1000 */
 
-# if	defined(DTYPE_LLA)
+# if	defined(DTYPE_LLA) && HPUXV<1030
 #define	_KERNEL	1
 #include <sio/lla.h>
 #undef	_KERNEL
-# endif
+# endif	/* defined(DTYPE_LLA) && HPUXV<1030 */
 
 #include <sys/un.h>
 #include <sys/unpcb.h>
@@ -165,7 +225,11 @@ struct mount {
 # endif	/* HPUXV>=800 */
 
 #define DEVINCR		1024	/* device table malloc() increment */
+
+# if	!defined(KA_T)
 #define	KA_T		off_t
+# endif	/* !defined(KA_T) */
+
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		void
@@ -202,6 +266,11 @@ struct mount {
 #define	U_SIZE		sizeof(struct user)
 # endif	/* HPUXV>=800 */
 
+# if	HPUXV>=1030
+#define	XDR_PMAPLIST	(xdrproc_t)xdr_pmaplist
+#define	XDR_VOID	(xdrproc_t)xdr_void
+# endif	/* HPUXV>=1030 */
+
 
 # if	defined(HAS_AFS)
 /*
@@ -228,11 +297,11 @@ extern int Kd;
 extern KA_T Kpa;
 
 # if	HPUXV>=1010
-extern caddr_t Ktp;
+extern KA_T Ktp;
 #endif	/* HPUXV>=1010 */
 
 struct l_vfs {
-	struct vfs *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
@@ -298,7 +367,7 @@ extern struct pte *Usrptmap;
 extern struct pte *usrpt;
 # endif	/* HPUXV<800 && defined(hp9000s300) */
 
-extern struct fileops *Vnfops;
+extern KA_T Vnfops;
 
 
 /*
