@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.1 97/02/24 07:33:51 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.3 98/03/06 08:21:28 abe Exp $";
 #endif
 
 
@@ -67,17 +67,28 @@ static char *rcsid = "$Id: dnode1.c,v 1.1 97/02/24 07:33:51 abe Exp $";
 #undef	i_size
 #undef	di_size
 #define	pool_id_t	vx_pool_id_t
+
+# if	HPUXV>=1030
+#define	ulong	vx_ulong		/* avoid <sys/stream.h> conflict */
+# endif	/* HPUXV>=1030 */
+
 #include <sys/fs/vx_hpux.h>
 #include <sys/fs/vx_port.h>
 #include <sys/fs/vx_inode.h>
+
+# if	HPUXV>=1030
+#undef	ulong
+# endif	/* HPUXV>=1030 */
+
 # else	/* HPUXV<1020 */
+
 #define	pool_id_t	caddr_t
 #define	sv_sema_t	caddr_t
 #define	_KERNEL
 #include <sys/fs/vx_hpux.h>
 #include <sys/fs/vx_inode.h>
 #undef	_KERNEL
-# endif	HPUXV>=1020
+# endif	/* HPUXV>=1020 */
 
 
 /*
@@ -86,23 +97,23 @@ static char *rcsid = "$Id: dnode1.c,v 1.1 97/02/24 07:33:51 abe Exp $";
 
 int
 read_vxnode(v, vfs, dev)
-	struct vnode *v;		/* containing vnode */
+	struct vnode *v;		/* local containing vnode */
 	struct l_vfs *vfs;		/* local vfs structure */
 	dev_t *dev;			/* device number receiver */
 {
 	struct vx_inode i;
 
 	if (!v->v_data || kread((KA_T)v->v_data, (char *)&i, sizeof(i)))
-		return(1);
+	    return(1);
 /*
  * Return device number.
  */
 	if (v->v_type == VCHR || v->v_type == VBLK)
-		*dev = v->v_rdev;
+	    *dev = v->v_rdev;
 	else if (vfs && vfs->fsname)
-		*dev = vfs->dev;
+	    *dev = vfs->dev;
 	else
-		*dev = i.i_dev;
+	    *dev = i.i_dev;
 /*
  * Record inode number.
  */
@@ -112,8 +123,8 @@ read_vxnode(v, vfs, dev)
  * Record size.
  */
 	if (!Foffset) {
-		Lf->sz = (unsigned long)i.i_size;
-		Lf->sz_def = 1;
+	    Lf->sz = (SZOFFTYPE)i.i_size;
+	    Lf->sz_def = 1;
 	}
 	return(0);
 }
