@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.9 98/02/16 19:51:59 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.11 98/03/06 14:44:19 abe Exp $";
 #endif
 
 
@@ -64,15 +64,15 @@ getmemsz(pid)
 	struct vmspace vm;
 
 	for (n = 0, p = P; n < Np; n++, p++) {
-		if (p->P_PID == pid) {
-			if (p->P_VMSPACE == NULL
-			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
-				return;
-			Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
-							     + vm.vm_ssize);
-			Lf->sz_def = 1;
-			return;
-		}
+	    if (p->P_PID == pid) {
+		if (!p->P_VMSPACE
+		||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
+		    return;
+		Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
+						     + vm.vm_ssize);
+		Lf->sz_def = 1;
+		return;
+	    }
 	}
 }
 #undef	PGSHIFT
@@ -131,11 +131,10 @@ lkup_dev_tty_again:
 
 void
 process_node(va)
-	caddr_t va;			/* vnode kernel space address */
+	KA_T va;			/* vnode kernel space address */
 {
 
 	dev_t dev;
-	char dev_ch[32];
 	int devs = 0;
 	struct lockf lf, *lff, *lfp;
 	int lt;
@@ -187,15 +186,15 @@ process_node(va)
 /*
  * Read the vnode.
  */
-	if ( ! va) {
-		enter_nm("no vnode address");
-		return;
+	if (!va) {
+	    enter_nm("no vnode address");
+	    return;
 	}
 	v = &vb;
-	if (readvnode((caddr_t)va, v)) {
-                enter_nm(Namech);
-                return;
-        }
+	if (readvnode(va, v)) {
+	    enter_nm(Namech);
+	    return;
+	}
 
 # if	defined(HASNCACHE)
 	Lf->id = v->v_id;
@@ -205,37 +204,37 @@ process_node(va)
 /*
  * Get the vnode type.
  */
-	if (v->v_mount == NULL)
-		vfs = NULL;
+	if (!v->v_mount)
+	    vfs = (struct l_vfs *)NULL;
 	else {
-		vfs = readvfs(v->v_mount);
-		if (vfs) {
-			if (strcmp(vfs->type, MOUNT_NFS) == 0)
-				Ntype = N_NFS;
+	    vfs = readvfs((KA_T)v->v_mount);
+	    if (vfs) {
+		if (strcmp(vfs->type, MOUNT_NFS) == 0)
+		    Ntype = N_NFS;
 
 #if	defined(HASKERNFS)
-			else if (strcmp(vfs->type, MOUNT_KERNFS) == 0)
-				Ntype = N_KERN;
+		else if (strcmp(vfs->type, MOUNT_KERNFS) == 0)
+		    Ntype = N_KERN;
 #endif	/* defined(HASKERNFS) */
 
 #if	defined(HASPROCFS)
-			else if (strcmp(vfs->type, MOUNT_PROCFS) == 0)
-				Ntype = N_PROC;
+		else if (strcmp(vfs->type, MOUNT_PROCFS) == 0)
+		    Ntype = N_PROC;
 #endif	/* defined(HASPROCFS) */
 
 #if	defined(HAS9660FS)
-			else if (strcmp(vfs->type, MOUNT_CD9660) == 0)
-				Ntype = N_CDFS;
+		else if (strcmp(vfs->type, MOUNT_CD9660) == 0)
+		    Ntype = N_CDFS;
 #endif	/* defined(HAS9660FS) */
 
-		}
+	    }
 	}
 	if (Ntype == N_REGLR) {
-		switch (v->v_type) {
-		case VFIFO:
-			Ntype = N_FIFO;
-			break;
-		}
+	    switch (v->v_type) {
+	    case VFIFO:
+		Ntype = N_FIFO;
+		break;
+	    }
 	}
 /*
  * Read the successor node.
@@ -244,17 +243,17 @@ process_node(va)
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
-		if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
-				  (SZOFFTYPE *)&iso_sz))
-		{
-			(void) sprintf(Namech, "can't read iso_node at: %x",
-			    v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-		iso_stat = 1;
-		nty = CDFSNODE;
-		break;
+	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
+		  (SZOFFTYPE *)&iso_sz))
+	    {
+		(void) sprintf(Namech, "can't read iso_node at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    iso_stat = 1;
+	    nty = CDFSNODE;
+	    break;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASKERNFS)
@@ -263,109 +262,109 @@ process_node(va)
 	/*
 	 * Read the kernfs_node.
 	 */
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&kn, sizeof(kn))) {
-		    if (v->v_type != VDIR || !(v->v_flag && VROOT)) {
-			(void) sprintf(Namech, "can't read kernfs_node at: %x",
-			    v->v_data);
-			enter_nm(Namech);
-			return;
-		    } else
-			kn.kf_kt = (struct kern_target *)NULL;
-		}
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&kn, sizeof(kn))) {
+		if (v->v_type != VDIR || !(v->v_flag && VROOT)) {
+		    (void) sprintf(Namech, "can't read kernfs_node at: %s",
+			print_kptr((KA_T)v->v_data, (char *)NULL));
+		    enter_nm(Namech);
+		    return;
+		} else
+		    kn.kf_kt = (struct kern_target *)NULL;
+	    }
 	/*
 	 * Generate the /kern file name by reading the kern_target to which
 	 * the kernfs_node points.
 	 */
-		if (kn.kf_kt
-		&&  kread((KA_T)kn.kf_kt, (char *)&kt, sizeof(kt)) == 0
-		&&  (ktnl = (int)kt.kt_namlen) > 0
-		&&  kt.kt_name)
-		{
-		    if (ktnl > (sizeof(ktnm) - 1))
-			ktnl = sizeof(ktnm) - 1;
-		    if (kread((KA_T)kt.kt_name, ktnm, ktnl) == 0) {
-			ktnm[ktnl] = 0;
-			ktnl = strlen(ktnm);
-			if (ktnl > (MAXPATHLEN - strlen(_PATH_KERNFS) - 2)) {
-			    ktnl = MAXPATHLEN - strlen(_PATH_KERNFS) - 2;
-			    ktnm[ktnl] = '\0';
-			}
-			(void) sprintf(Namech, "%s/%s", _PATH_KERNFS, ktnm);
+	    if (kn.kf_kt
+	    &&  kread((KA_T)kn.kf_kt, (char *)&kt, sizeof(kt)) == 0
+	    &&  (ktnl = (int)kt.kt_namlen) > 0
+	    &&  kt.kt_name)
+	    {
+		if (ktnl > (sizeof(ktnm) - 1))
+		    ktnl = sizeof(ktnm) - 1;
+		if (!kread((KA_T)kt.kt_name, ktnm, ktnl)) {
+		    ktnm[ktnl] = 0;
+		    ktnl = strlen(ktnm);
+		    if (ktnl > (MAXPATHLEN - strlen(_PATH_KERNFS) - 2)) {
+			ktnl = MAXPATHLEN - strlen(_PATH_KERNFS) - 2;
+			ktnm[ktnl] = '\0';
 		    }
+		    (void) sprintf(Namech, "%s/%s", _PATH_KERNFS, ktnm);
 		}
-	    /*
-	     * If this is the /kern root directory, its name, inode number and
-	     * size are fixed; otherwise, safely stat() the file to get the
-	     * inode number and size.
-	     */
-		if (v->v_type == VDIR && (v->v_flag & VROOT)) {
-		    (void) strcpy(Namech, _PATH_KERNFS);
-		    ksb.st_ino = 2;
-		    ksb.st_size = DEV_BSIZE;
-		    ksbs = 1;
-		} else if (Namech[0] && statsafely(Namech, &ksb) == 0)
-		    ksbs = 1;
-		nty = KERNFSNODE;
-		break;
+	    }
+	/*
+	 * If this is the /kern root directory, its name, inode number and
+	 * size are fixed; otherwise, safely stat() the file to get the
+	 * inode number and size.
+	 */
+	    if (v->v_type == VDIR && (v->v_flag & VROOT)) {
+		(void) strcpy(Namech, _PATH_KERNFS);
+		ksb.st_ino = 2;
+		ksb.st_size = DEV_BSIZE;
+		ksbs = 1;
+	    } else if (Namech[0] && statsafely(Namech, &ksb) == 0)
+		ksbs = 1;
+	    nty = KERNFSNODE;
+	    break;
 #endif	/* defined(HASKERNFS) */
 
 	case VT_MFS:
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&m, sizeof(m))) {
-			(void) sprintf(Namech, "can't read mfsnode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-		nty = MFSNODE;
-		break;
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&m, sizeof(m))) {
+		(void) sprintf(Namech, "can't read mfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = MFSNODE;
+	    break;
 	case VT_NFS:
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
-			(void) sprintf(Namech, "can't read nfsnode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
+		(void) sprintf(Namech, "can't read nfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
 
 #if	defined(HASNFSVATTRP)
-		if (!n.n_vattr
-		||  kread((KA_T)n.n_vattr, (char *)&nv, sizeof(nv))) {
-			(void) sprintf(Namech, "can't read n_vattr at: %x",
-				n.n_vattr);
-			enter_nm(Namech);
-			return;
-		}
+	    if (!n.n_vattr
+	    ||  kread((KA_T)n.n_vattr, (char *)&nv, sizeof(nv))) {
+		(void) sprintf(Namech, "can't read n_vattr at: %x",
+		    print_kptr((KA_T)n.n_vattr, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
 #endif	/* defined(HASNFSVATTRP) */
 
-		nty = NFSNODE;
-		break;
+	    nty = NFSNODE;
+	    break;
 
 #if	defined(HASFDESCFS)
 	case VT_FDESC:
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
-			(void) sprintf(Namech, "can't read fdescnode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-		nty = FDESCNODE;
-		break;
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
+		(void) sprintf(Namech, "can't read fdescnode at: %x",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = FDESCNODE;
+	    break;
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&p, sizeof(p))) {
-			(void) sprintf(Namech, "can't read pfsnode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-		nty = PFSNODE;
-		break;
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&p, sizeof(p))) {
+		(void) sprintf(Namech, "can't read pfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = PFSNODE;
+	    break;
 #endif	/* defined(HASPROCFS) */
 
 #if	defined(HASEXT2FS)
@@ -373,65 +372,65 @@ process_node(va)
 #endif	/* defined(HASEXT2FS) */
 
 	case VT_UFS:
-		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
-			(void) sprintf(Namech, "can't read inode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-
-#if	defined(HASEXT2FS)
-		if (v->v_tag == VT_EXT2FS)
-			nty = EXT2NODE;
-		else
-#endif	/* defined(HASEXT2FS) */
-
-			nty = INODE;
-		if ((lff = i.i_lockf)) {
-
-		/*
-		 * Determine the lock state.
-		 */
-		    lfp = lff;
-		    do {
-			if (kread((KA_T)lfp, (char *)&lf, sizeof(lf)))
-			    break;
-			lt = 0;
-			switch(lf.lf_flags & (F_FLOCK|F_POSIX)) {
-			case F_FLOCK:
-			    if (Cfp && (struct file *)lf.lf_id == Cfp)
-				lt = 1;
-			    break;
-			case F_POSIX:
-			    if ((KA_T)lf.lf_id == Kpa)
-				lt = 1;
-			    break;
-			}
-			if (!lt)
-			    continue;
-			if (lf.lf_start == (off_t)0
-			&&  lf.lf_end == 0xffffffffffffffffLL)
-			    lt = 1;
-			else
-			    lt = 0;
-			if (lf.lf_type == F_RDLCK)
-			    Lf->lock = lt ? 'R' : 'r';
-			else if (lf.lf_type == F_WRLCK)
-			    Lf->lock = lt ? 'W' : 'w';
-			else if (lf.lf_type == (F_RDLCK | F_WRLCK))
-			    Lf->lock = 'u';
-			break;
-		    } while ((lfp = lf.lf_next) && lfp != lff);
-		}
-		break;
-	default:
-		if (v->v_type == VBAD || v->v_type == VNON)
-			break;
-		(void) sprintf(Namech, "unknown file system type: %d",
-			v->v_tag);
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
+		(void) sprintf(Namech, "can't read inode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
+	    }
+
+#if	defined(HASEXT2FS)
+	    if (v->v_tag == VT_EXT2FS)
+		nty = EXT2NODE;
+	    else
+#endif	/* defined(HASEXT2FS) */
+
+		nty = INODE;
+	    if ((lff = i.i_lockf)) {
+
+	    /*
+	     * Determine the lock state.
+	     */
+		lfp = lff;
+		do {
+		    if (kread((KA_T)lfp, (char *)&lf, sizeof(lf)))
+			break;
+		    lt = 0;
+		    switch(lf.lf_flags & (F_FLOCK|F_POSIX)) {
+		    case F_FLOCK:
+			if (Cfp && (struct file *)lf.lf_id == Cfp)
+			    lt = 1;
+			break;
+		    case F_POSIX:
+			if ((KA_T)lf.lf_id == Kpa)
+			    lt = 1;
+			break;
+		    }
+		    if (!lt)
+			continue;
+		    if (lf.lf_start == (off_t)0
+		    &&  lf.lf_end == 0xffffffffffffffffLL)
+			lt = 1;
+		    else
+			lt = 0;
+		    if (lf.lf_type == F_RDLCK)
+			Lf->lock = lt ? 'R' : 'r';
+		    else if (lf.lf_type == F_WRLCK)
+			Lf->lock = lt ? 'W' : 'w';
+		    else if (lf.lf_type == (F_RDLCK | F_WRLCK))
+			Lf->lock = 'u';
+		    break;
+		} while ((lfp = lf.lf_next) && lfp != lff);
+	    }
+	    break;
+	default:
+	    if (v->v_type == VBAD || v->v_type == VNON)
+		break;
+	    (void) sprintf(Namech, "unknown file system type: %d",
+		v->v_tag);
+	    enter_nm(Namech);
+	    return;
 	}
 /*
  * Get device and type for printing.
@@ -441,70 +440,70 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
-		if (f.fd_link
-		&&  kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1) == 0)
-		{
-		    Namech[sizeof(Namech) - 1] = '\0';
-		    break;
-		}
-		if (f.fd_type == Fctty) {
-		    if (f_tty_s == 0)
-			f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
-		    if (f_tty_s == 1) {
-			dev = f_tty_dev;
-			Lf->inode = f_tty_ino;
-			devs = Lf->inp_ty = 1;
-		    }
-		}
+	    if (f.fd_link
+	    &&  !kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1)) {
+		Namech[sizeof(Namech) - 1] = '\0';
 		break;
+	    }
+	    if (f.fd_type == Fctty) {
+		if (f_tty_s == 0)
+		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
+		if (f_tty_s == 1) {
+		    dev = f_tty_dev;
+		    Lf->inode = f_tty_ino;
+		    devs = Lf->inp_ty = 1;
+		}
+	    }
+	    break;
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASEXT2FS)
 	case EXT2NODE:
 
 # if	defined(HASI_FFS)
-		dev = i.i_dev;
+	    dev = i.i_dev;
 # else	/* !defined(HASI_FFS) */
-		dev = (type == VCHR || type == VBLK) ? i.i_rdev : i.i_dev;
-		break;
+	    dev = (type == VCHR || type == VBLK) ? i.i_rdev : i.i_dev;
 # endif	/* defined(HASI_FFS) */
+
+	    break;
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-		dev = (type == VCHR || type == VBLK) ?
+	    dev = (type == VCHR || type == VBLK) ?
 
 #if	defined(HASI_FFS)
-						       i.i_ffs_rdev
+						   i.i_ffs_rdev
 #else	/* !defined(HASI_FFS) */
-						       i.i_rdev
+						   i.i_rdev
 #endif	/* defined(HASI_FFS) */
 
-						     : i.i_dev;
-		devs = 1;
-		break;
+						 : i.i_dev;
+	    devs = 1;
+	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
-		if (vfs) {
-			dev = (dev_t)vfs->fsid.val[0];
-			devs = 1;
-		}
-		break;
+	    if (vfs) {
+		dev = (dev_t)vfs->fsid.val[0];
+		devs = 1;
+	    }
+	    break;
 #endif	/* defined(HASKERNFS) */
 
 
 #if	defined(HAS9660FS)
 	case CDFSNODE:
-		if (iso_stat) {
-			dev = iso_dev;
-			devs = 1;
-		}
-		break;
+	    if (iso_stat) {
+		dev = iso_dev;
+		devs = 1;
+	    }
+	    break;
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-		dev = NVATTR.va_fsid;
-		devs = 1;
+	    dev = NVATTR.va_fsid;
+	    devs = 1;
 	}
 /*
  * Obtain the inode number.
@@ -516,40 +515,40 @@ process_node(va)
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-		if (type != VBLK) {
-			Lf->inode = (unsigned long)i.i_number;
-			Lf->inp_ty = 1;
-		}
-		break;
+	    if (type != VBLK) {
+		Lf->inode = (unsigned long)i.i_number;
+		Lf->inp_ty = 1;
+	    }
+	    break;
 
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
-		if (ksbs) {
-			Lf->inode = (unsigned long)ksb.st_ino;
-			Lf->inp_ty = 1;
-		}
-		break;
+	    if (ksbs) {
+		Lf->inode = (unsigned long)ksb.st_ino;
+		Lf->inp_ty = 1;
+	    }
+	    break;
 #endif	/* defined(HASKERNFS) */
 
 #if	defined(HAS9660FS)
 	case CDFSNODE:
-		if (iso_stat) {
-			Lf->inode = iso_ino;
-			Lf->inp_ty = 1;
-		}
-		break;
+	    if (iso_stat) {
+		Lf->inode = iso_ino;
+		Lf->inp_ty = 1;
+	    }
+	    break;
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-		Lf->inode = (unsigned long)NVATTR.va_fileid;
-		Lf->inp_ty = 1;
-		break;
+	    Lf->inode = (unsigned long)NVATTR.va_fileid;
+	    Lf->inp_ty = 1;
+	    break;
 
 #if	defined(HASPROCFS)
 	case PFSNODE:
-		Lf->inode = (unsigned long)p.pfs_fileno;
-		Lf->inp_ty = 1;
-		break;
+	    Lf->inode = (unsigned long)p.pfs_fileno;
+	    Lf->inp_ty = 1;
+	    break;
 #endif	/* defined(HASPROCFS) */
 
 	}
@@ -558,182 +557,181 @@ process_node(va)
  * Obtain the file size.
  */
 	if (Foffset)
-		Lf->off_def = 1;
+	    Lf->off_def = 1;
 	else {
-		switch (Ntype) {
+	    switch (Ntype) {
 
 #if	defined(HAS9660FS)
-		case N_CDFS:
-			if (iso_stat) {
-				Lf->sz = (SZOFFTYPE)iso_sz;
-				Lf->sz_def = 1;
-			}
-			break;
+	    case N_CDFS:
+		if (iso_stat) {
+		    Lf->sz = (SZOFFTYPE)iso_sz;
+		    Lf->sz_def = 1;
+		}
+		break;
 #endif	/* defined(HAS9660FS) */
 
-		case N_FIFO:
-			if (!Fsize)
-				Lf->off_def = 1;
-			break;
+	    case N_FIFO:
+		if (!Fsize)
+		    Lf->off_def = 1;
+		break;
 
 #if	defined(HASKERNFS)
-		case N_KERN:
-			if (ksbs) {
-				Lf->sz = (SZOFFTYPE)ksb.st_size;
-				Lf->sz_def = 1;
-			}
-			break;
+	    case N_KERN:
+		if (ksbs) {
+		    Lf->sz = (SZOFFTYPE)ksb.st_size;
+		    Lf->sz_def = 1;
+		}
+		break;
 #endif	/* defined(HASKERNFS) */
 
-		case N_NFS:
-			if (nty == NFSNODE) {
-				Lf->sz = (SZOFFTYPE)NVATTR.va_size;
-				Lf->sz_def = 1;
-			}
-			break;
+	    case N_NFS:
+		if (nty == NFSNODE) {
+		    Lf->sz = (SZOFFTYPE)NVATTR.va_size;
+		    Lf->sz_def = 1;
+		}
+		break;
 
 #if	defined(HASPROCFS)
-		case N_PROC:
-			if (nty == PFSNODE) {
-				switch (p.pfs_type) {
-				case Proot:
-				case Pproc:
-					Lf->sz = (SZOFFTYPE)DEV_BSIZE;
-					Lf->sz_def = 1;
-					break;
-				case Pcurproc:
-					Lf->sz = (SZOFFTYPE)DEV_BSIZE;
-					Lf->sz_def = 1;
-					break;
-				case Pmem:
-					(void) getmemsz(p.pfs_pid);
-					break;
-				case Pregs:
-					Lf->sz = (SZOFFTYPE)sizeof(struct reg);
-					Lf->sz_def = 1;
-					break;
+	    case N_PROC:
+		if (nty == PFSNODE) {
+		    switch (p.pfs_type) {
+		    case Proot:
+		    case Pproc:
+			Lf->sz = (SZOFFTYPE)DEV_BSIZE;
+			Lf->sz_def = 1;
+			break;
+		    case Pcurproc:
+			Lf->sz = (SZOFFTYPE)DEV_BSIZE;
+			Lf->sz_def = 1;
+			break;
+		    case Pmem:
+			(void) getmemsz(p.pfs_pid);
+			break;
+		    case Pregs:
+			Lf->sz = (SZOFFTYPE)sizeof(struct reg);
+			Lf->sz_def = 1;
+			break;
 
 # if	defined(FP_QSIZE)
-				case Pfpregs:
-					Lf->sz = (SZOFFTYPE)sizeof(struct fpreg);
-					Lf->sz_def = 1;
-					break;
+		    case Pfpregs:
+			Lf->sz = (SZOFFTYPE)sizeof(struct fpreg);
+			Lf->sz_def = 1;
+			break;
 # endif	/* defined(FP_QSIZE) */
 
-				}
-			}
-			break;
+		    }
+		}
+		break;
 #endif	/* defined(HASPROCFS) */
 
-		case N_REGLR:
-			if (type == VREG || type == VDIR) {
-				if (nty == INODE) {
+	    case N_REGLR:
+		if (type == VREG || type == VDIR) {
+		    if (nty == INODE) {
 
 #if	defined(HASI_FFS)
-					Lf->sz = (SZOFFTYPE)i.i_ffs_size;
+			Lf->sz = (SZOFFTYPE)i.i_ffs_size;
 #else	/* !defined(HASI_FFS) */
-					Lf->sz = (SZOFFTYPE)i.i_size;
+			Lf->sz = (SZOFFTYPE)i.i_size;
 #endif	/* defined(HASI_FFS) */
 
-					Lf->sz_def = 1;
-				} else if (nty == MFSNODE) {
-					Lf->sz = (SZOFFTYPE)m.mfs_size;
-					Lf->sz_def = 1;
-				}
+			Lf->sz_def = 1;
+		    } else if (nty == MFSNODE) {
+			Lf->sz = (SZOFFTYPE)m.mfs_size;
+			Lf->sz_def = 1;
+		    }
 
 #if	defined(HASEXT2FS)
-				else if (nty == EXT2NODE) {
+		    else if (nty == EXT2NODE) {
 
 # if	defined(HASI_E2FS)
-					Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
+			Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
 # else	/* !defined(HASI_E2FS) */
-					Lf->sz = (SZOFFTYPE)i.i_size;
+			Lf->sz = (SZOFFTYPE)i.i_size;
 # endif	/* defined(HASI_E2FS) */
 
-					Lf->sz_def = 1;
-				}
+			Lf->sz_def = 1;
+		    }
 #endif	/* defined(HASEXT2FS) */
 
-			}
-			else if ((type == VCHR || type == VBLK) && !Fsize)
-				Lf->off_def = 1;
-			break;
 		}
+		else if ((type == VCHR || type == VBLK) && !Fsize)
+		    Lf->off_def = 1;
+		break;
+	    }
 	}
 /*
  * Record an NFS file selection.
  */
 	if (Ntype == N_NFS && Fnfs)
-		Lf->sf |= SELNFS;
+	    Lf->sf |= SELNFS;
 /*
  * Save the file system names.
  */
 	if (vfs) {
-		Lf->fsdir = vfs->dir;
-		Lf->fsdev = vfs->fsname;
+	    Lf->fsdir = vfs->dir;
+	    Lf->fsdev = vfs->fsname;
 	}
 /*
  * Format the vnode type, and possibly the device name.
  */
 	switch (type) {
-
 	case VNON:
-		ty ="VNON";
-		break;
+	    ty ="VNON";
+	    break;
 	case VREG:
 	case VDIR:
-		ty = (type == VREG) ? "VREG" : "VDIR";
-		if (devs) {
-			Lf->dev_def = 1;
-			Lf->dev = dev;
-		}
-		break;
+	    ty = (type == VREG) ? "VREG" : "VDIR";
+	    if (devs) {
+		Lf->dev_def = 1;
+		Lf->dev = dev;
+	    }
+	    break;
 	case VBLK:
-		ty = "VBLK";
-		if (devs) {
-			Lf->dev_def = 1;
-			Lf->dev = dev;
-		}
-		Ntype = N_BLK;
-		break;
+	    ty = "VBLK";
+	    if (devs) {
+		Lf->dev_def = 1;
+		Lf->dev = dev;
+	    }
+	    Ntype = N_BLK;
+	    break;
 	case VCHR:
-		ty = "VCHR";
-		if (devs) {
-			Lf->dev = dev;
-			Lf->dev_def = 1;
-		}
-		Ntype = N_CHR;
-		break;
+	    ty = "VCHR";
+	    if (devs) {
+		Lf->dev = dev;
+		Lf->dev_def = 1;
+	    }
+	    Ntype = N_CHR;
+	    break;
 	case VLNK:
-		ty = "VLNK";
-		break;
+	    ty = "VLNK";
+	    break;
 
 #if	defined(VSOCK)
 	case VSOCK:
-		ty = "SOCK";
-		break;
+	    ty = "SOCK";
+	    break;
 #endif	/* defined(VSOCK) */
 
 	case VBAD:
-		ty = "VBAD";
-		break;
+	    ty = "VBAD";
+	    break;
 	case VFIFO:
-		if (devs) {
-			Lf->dev = dev;
-			Lf->dev_def = 1;
-		}
-		ty = "FIFO";
-		break;
+	    if (devs) {
+		Lf->dev = dev;
+		Lf->dev_def = 1;
+	    }
+	    ty = "FIFO";
+	    break;
 	default:
-		if (type > 9999)
-			(void) sprintf(Lf->type, "*%03d", type % 1000);
-		else
-			(void) sprintf(Lf->type, "%4d", type);
-		(void) strcpy(Namech, "unknown type");
-		ty = NULL;
+	    if (type > 9999)
+		(void) sprintf(Lf->type, "*%03d", type % 1000);
+	    else
+		(void) sprintf(Lf->type, "%4d", type);
+	    (void) strcpy(Namech, "unknown type");
+	    ty = NULL;
 	}
 	if (ty)
-		(void) strcpy(Lf->type, ty);
+	    (void) strcpy(Lf->type, ty);
 	Lf->ntype = Ntype;
 /*
  * Handle some special cases:
@@ -745,66 +743,66 @@ process_node(va)
  */
 
 	if (type == VBAD)
-		(void) strcpy(Namech, "(revoked)");
+	    (void) strcpy(Namech, "(revoked)");
 	else if (nty == MFSNODE) {
-		Lf->dev_def = 0;
-		(void) sprintf(Namech, "%#x", m.mfs_baseoff);
-		(void) strcpy(dev_ch, "    memory");
-		enter_dev_ch(dev_ch);
+	    Lf->dev_def = 0;
+	    (void) sprintf(Namech, "%#x", m.mfs_baseoff);
+	    enter_dev_ch("memory");
 	}
 
 #if	defined(HASPROCFS)
 	else if (nty == PFSNODE) {
-		Lf->dev_def = 0;
-		ty = NULL;
-		(void) sprintf(Namech, "/%s", HASPROCFS);
-		switch (p.pfs_type) {
-		case Proot:
-			ty = "PDIR";
-			break;
-		case Pcurproc:
-			(void) strcpy(endnm(), "/curproc");
-			ty = "PCUR";
-			break;
-		case Pproc:
-			(void) sprintf(endnm(), "/%d", p.pfs_pid);
-			ty = "PDIR";
-			break;
-		case Pfile:
-			(void) sprintf(endnm(), "/%d/file", p.pfs_pid);
-			ty = "PFIL";
-			break;
-		case Pmem:
-			(void) sprintf(endnm(), "/%d/mem", p.pfs_pid);
-			ty = "PMEM";
-			break;
-		case Pregs:
-			(void) sprintf(endnm(), "/%d/regs", p.pfs_pid);
-			ty = "PREG";
-			break;
-		case Pfpregs:
-			(void) sprintf(endnm(), "/%d/fpregs", p.pfs_pid);
-			ty = "PFPR";
-			break;
-		case Pctl:
-			(void) sprintf(endnm(), "/%d/ctl", p.pfs_pid);
-			ty = "PCTL";
-			break;
-		case Pstatus:
-			(void) sprintf(endnm(), "/%d/status", p.pfs_pid);
-			ty = "PSTA";
-			break;
-		case Pnote:
-			(void) sprintf(endnm(), "/%d/note", p.pfs_pid);
-			ty = "PNTF";
-			break;
-		case Pnotepg:
-			(void) sprintf(endnm(), "/%d/notepg", p.pfs_pid);
-			ty = "PGRP";
-			break;
-		}
-		if (ty)
-			(void) strcpy(Lf->type, ty);
+	    Lf->dev_def = 0;
+	    ty = NULL;
+	    (void) sprintf(Namech, "/%s", HASPROCFS);
+	    switch (p.pfs_type) {
+	    case Proot:
+		ty = "PDIR";
+		break;
+	    case Pcurproc:
+		(void) strcpy(endnm(), "/curproc");
+		ty = "PCUR";
+		break;
+	    case Pproc:
+		(void) sprintf(endnm(), "/%d", p.pfs_pid);
+		ty = "PDIR";
+		break;
+	    case Pfile:
+		(void) sprintf(endnm(), "/%d/file", p.pfs_pid);
+		ty = "PFIL";
+		break;
+	    case Pmem:
+		(void) sprintf(endnm(), "/%d/mem", p.pfs_pid);
+		ty = "PMEM";
+		break;
+	    case Pregs:
+		(void) sprintf(endnm(), "/%d/regs", p.pfs_pid);
+		ty = "PREG";
+		break;
+	    case Pfpregs:
+		(void) sprintf(endnm(), "/%d/fpregs", p.pfs_pid);
+		ty = "PFPR";
+		break;
+	    case Pctl:
+		(void) sprintf(endnm(), "/%d/ctl", p.pfs_pid);
+		ty = "PCTL";
+		break;
+	    case Pstatus:
+		(void) sprintf(endnm(), "/%d/status", p.pfs_pid);
+		ty = "PSTA";
+		break;
+	    case Pnote:
+		(void) sprintf(endnm(), "/%d/note", p.pfs_pid);
+		ty = "PNTF";
+		break;
+	    case Pnotepg:
+		(void) sprintf(endnm(), "/%d/notepg", p.pfs_pid);
+		ty = "PGRP";
+		break;
+	    }
+	    if (ty)
+		(void) strcpy(Lf->type, ty);
+	    if (Namech[0])
 		enter_nm(Namech);
 	}
 #endif	/* defined(HASPROCFS) */
@@ -815,7 +813,7 @@ process_node(va)
  * supply one.
  */
 	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
-		find_bl_ino();
+	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
 /*
@@ -823,7 +821,7 @@ process_node(va)
  * supply one.
  */
 	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
-		find_ch_ino();
+	    find_ch_ino();
 /*
  * Test for specified file.
  */
@@ -856,21 +854,21 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	{
-		if (Namech[0]) {
-			enter_nm(Namech);
-			ns = 1;
-		} else
-			ns = 0;
-		if (Sfile && is_file_named(NULL, type))
-			Lf->sf |= SELNM;
-		if (ns)
-			Namech[0] = '\0';
+	    if (Namech[0]) {
+		enter_nm(Namech);
+		ns = 1;
+	    } else
+		ns = 0;
+	    if (Sfile && is_file_named(NULL, type))
+		Lf->sf |= SELNM;
+	    if (ns)
+		Namech[0] = '\0';
 	}
 /*
  * Enter name characters.
  */
 	if (Namech[0])
-		enter_nm(Namech);
+	    enter_nm(Namech);
 }
 
 
@@ -881,40 +879,39 @@ process_node(va)
 
 void
 process_pipe(pa)
-	caddr_t pa;			/* pipe structure address */
+	KA_T pa;			/* pipe structure kernel address */
 {
-	char dev_ch[32];
 	struct pipe p;
 
 	if (!pa || kread((KA_T)pa, (char *)&p, sizeof(p))) {
-		(void) sprintf(Namech,
-			"can't read DTYPE_PIPE pipe struct: %#x", pa);
-		enter_nm(Namech);
-		return;
+	    (void) sprintf(Namech, "can't read DTYPE_PIPE pipe struct: %#s",
+		print_kptr(pa, (char *)NULL));
+	    enter_nm(Namech);
+	    return;
 	}
 	(void) strcpy(Lf->type, "PIPE");
-	(void) sprintf(dev_ch, "%#x", pa);
-	enter_dev_ch(dev_ch);
+	enter_dev_ch(print_kptr(pa, (char *)NULL));
 	if (Foffset)
-		Lf->off_def = 1;
+	    Lf->off_def = 1;
 	else {
-		Lf->sz = (SZOFFTYPE)p.pipe_buffer.size;
-		Lf->sz_def = 1;
+	    Lf->sz = (SZOFFTYPE)p.pipe_buffer.size;
+	    Lf->sz_def = 1;
 	}
 	if (p.pipe_peer)
-		(void) sprintf(Namech, "->%#x", p.pipe_peer);
+	    (void) sprintf(Namech, "->%s",
+		print_kptr((KA_T)p.pipe_peer, (char *)NULL));
 	else
-		Namech[0] = '\0';
+	    Namech[0] = '\0';
 	if (p.pipe_buffer.cnt)
-		(void) sprintf(endnm(), "%, cnt=%d", p.pipe_buffer.cnt);
+	    (void) sprintf(endnm(), "%, cnt=%d", p.pipe_buffer.cnt);
 	if (p.pipe_buffer.in)
-		(void) sprintf(endnm(), "%, in=%d", p.pipe_buffer.in);
+	    (void) sprintf(endnm(), "%, in=%d", p.pipe_buffer.in);
 	if (p.pipe_buffer.out)
-		(void) sprintf(endnm(), "%, out=%d", p.pipe_buffer.out);
+	    (void) sprintf(endnm(), "%, out=%d", p.pipe_buffer.out);
 /*
  * Enter name characters.
  */
 	if (Namech[0])
-		enter_nm(Namech);
+	    enter_nm(Namech);
 }
 #endif	/* defined(OPENBSDV) */
