@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.6 98/02/13 12:44:00 abe Exp $
+ * $Id: dlsof.h,v 1.7 98/03/06 08:25:31 abe Exp $
  */
 
 
@@ -200,7 +200,7 @@ extern int HaveEventMajor;
 extern int HaveSockdev;
 extern int Hz;
 extern int Kd;
-extern unsigned long Lbolt;
+extern KA_T Lbolt;
 
 struct l_ino {
 	dev_t dev;			/* device */
