@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.11 98/02/16 19:53:48 abe Exp $
+ * $Id: machine.h,v 1.12 98/03/06 08:27:44 abe Exp $
  */
 
 
@@ -181,7 +181,7 @@
  * private lfile elements are used.
  */
 
-#define HASLFILEADD unsigned char fct_def, ina_def; long fct; caddr_t ina;
+#define HASLFILEADD unsigned char fct_def, ina_def; long fct; KA_T ina;
 #define SETLFILEADD Lf->fct_def = Lf->ina_def = 0;
 #define	HASFIELDAP1	"PTX link count"
 #define LISTLFILEAP1 \
