@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.4 97/07/02 11:15:26 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.6 98/03/06 08:38:13 abe Exp $";
 #endif
 
 
@@ -83,11 +83,11 @@ make_devtp(s, p, nl)
  * Store the device number, inode number, and name in the Devtp[] entry.
  */
 	Devtp[Devx].inode = s->st_ino;
-	if (!(Devtp[Devx].name = (char *)malloc(nl))) {
-	    (void) fprintf(stderr, "%s: no space for /dev/%s\n", Pn, p);
+	if (!(Devtp[Devx].name = mkstrcpy(p, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no space for /dev/", Pn);
+	    safestrprt(p, stderr, 1);
 	    Exit(1);
 	}
-	(void) strcpy(Devtp[Devx].name, p);
 	Devtp[Devx].rdev = s->st_rdev;
 	Devtp[Devx].v = 0;
 	Devx++;
@@ -122,10 +122,7 @@ printchdevname_again:
 #endif	/* defined(HASDCACHE) */
 
 	if ((dp = lkupdev(rdev, 1, 0))) {
-	    if (f)
-		(void) puts(dp->name);
-	    else
-		(void) fputs(dp->name, stdout);
+	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
 /*
@@ -136,16 +133,13 @@ printchdevname_again:
 		if (major(*rdev) == minor(c->cd.rdev)) {
 
 #if	defined(HASDCACHE)
-			if (DCunsafe && !c->cd.v && !vfy_dev(&c->cd))
-			    goto printchdevname_again;
+		    if (DCunsafe && !c->cd.v && !vfy_dev(&c->cd))
+		    goto printchdevname_again;
 #endif	/* defined(HASDCACHE) */
 
-			if (f)
-			    (void) puts(c->cd.name);
-			else
-			    (void) fputs(c->cd.name, stdout);
-			return(1);
-		}
+		    safestrprt(c->cd.name, stdout, f);
+		    return(1);
+		}	
 	    }
 	}
 
@@ -161,10 +155,7 @@ printchdevname_again:
 		    goto printchdevname_again;
 # endif	/* defined(HASDCACHE) */
 
-		if (f)
-		    (void) puts(p->pd.name);
-		else
-		    (void) fputs(p->pd.name, stdout);
+		safestrprt(p->pd.name, stdout, f);
 		return(1);
 	    }
 	}
@@ -197,31 +188,36 @@ read_clone()
 	char *cn;
 	DIR *dfp;
 	struct dirent *dp;
-	MALLOC_S nl;
-	char path[MAXNAMLEN+1];
+	char *fp = (char *)NULL;
+	MALLOC_S fpl;
+	char *path;
+	MALLOC_S pl;
 	struct pseudo *p;
-	int pl;
 	struct stat sb;
-	STRNCPY_L dnamel;
 	
 	if (Clone || Pseudo)
 	    return;
 /*
  * Open the /DVCH_DEVPATH/pseudo directory.
  */
-	(void) sprintf(path, "%s/pseudo", DVCH_DEVPATH);
+	if (!(path = mkstrcat(DVCH_DEVPATH, -1, "/", 1, "pseudo ", -1, &pl))) {
+	    (void) fprintf(stderr, "%s: no space for %s/pseudo\n", Pn);
+	    Exit(1);
+	}
+	path[pl - 1] = '\0';
 	if (!(dfp = opendir(path))) {
 
 #if	defined(WARNDEVACCESS)
-	    if (!Fwarn)
-		(void) fprintf(stderr, "%s: WARNING: can't open %s\n",
-		    Pn, path);
+	    if (!Fwarn) {
+		(void) fprintf(stderr, "%s: WARNING: can't open: ", Pn);
+		safestrprt(path, stderr, 1);
+	    }
 #endif	/* defined(WARNDEVACCESS) */
 
+	    (void) free((FREE_P *)path);
 	    return;
 	}
-	(void) strcat(path, "/");
-	pl = strlen(path);
+	path[pl - 1] = '/';
 /*
  * Scan the directory.
  */
@@ -231,27 +227,33 @@ read_clone()
 	/*
 	 * Form the full path name and stat() it.
 	 */
-	    dnamel = strlen(dp->d_name);
-	    if ((nl = pl + dnamel) >= sizeof(path)) {
-		if (!Fwarn)
-		    (void) fprintf(stderr,
-			"%s: /%s/pseudo entry name too long: %s\n",
-			Pn, DVCH_DEVPATH, dp->d_name);
-		continue;
+	    if (fp) {
+		(void) free((FREE_P *)fp);
+		fp = (char *)NULL;
+	    }
+	    if (!(fp = mkstrcat(path, pl, dp->d_name, -1, (char *)NULL, -1,
+		       &fpl)))
+	    {
+		(void) fprintf(stderr, "%s: no space for: ", Pn);
+		safestrprt(path, stderr, 0);
+		safestrprt(dp->d_name, stderr, 1);
+		Exit(1);
 	    }
-	    (void) strncpy(&path[pl], dp->d_name, dnamel);
-	    path[nl++] = '\0';
 
 #if	defined(USE_STAT)
-	    if (stat(path, &sb) != 0)
+	    if (stat(fp, &sb) != 0)
 #else	/* !defined(USE_STAT) */
-	    if (lstat(path, &sb) != 0)
+	    if (lstat(fp, &sb) != 0)
 #endif	/* defined(USE_STAT) */
 
 	    {
-		if (!Fwarn)
-		    (void) fprintf(stderr, "%s: can't stat %s: %s\n",
-			Pn, path, strerror(errno));
+		if (!Fwarn) {
+		    int errno_save = errno;
+
+		    (void) fprintf(stderr, "%s: can't stat: ", Pn);
+		    safestrprt(fp, stderr, 0);
+		    (void) fprintf(stderr, ": %s\n", strerror(errno_save));
+		}
 		continue;
 	    }
 	/*
@@ -263,38 +265,36 @@ read_clone()
 	/*
 	 * Make Devtp[] entry.
 	 */
-	    make_devtp(&sb, path, nl);
+	    make_devtp(&sb, fp, fpl);
 	 /*
 	  * Create a clone structure entry for "clone*:" devices.
 	  *
 	  * Make special note of network clones -- icmp, ip, tcp, and udp.
 	  */
-	    if (strncmp(&path[pl], "clone", 5) == 0) {
-		if (!(cn = strrchr(&path[pl], ':')))
-			    continue;
+	    if (strncmp(&fp[pl], "clone", 5) == 0) {
+		if (!(cn = strrchr(&fp[pl], ':')))
+		    continue;
 	    /*
 	     * Allocate a clone structure.
 	     */
 		if (!(c = (struct clone *)malloc(sizeof(struct clone)))) {
 		    (void) fprintf(stderr,
-			"%s: no space for network clone device: %s\n",
-			Pn, path);
+			"%s: no space for network clone device: ", Pn);
+		    safestrprt(fp, stderr, 1);
 		    Exit(1);
 		}
 	    /*
 	     * Allocate space for the path name.
 	     */
-		if (!(c->cd.name = (char *)malloc(nl))) {
-		    (void) fprintf(stderr,
-			"%s: no space for clone name: %s\n", Pn, path);
+		if (!(c->cd.name = mkstrcpy(fp, (MALLOC_S *)NULL))) {
+		    (void) fprintf(stderr, "%s: no space for clone name: ", Pn);
+		    safestrprt(fp, stderr, 1);
 		    Exit(1);
 		}
 	    /*
-	     * Save the path name, and inode and device numbers.  Clear the
-	     * verify flag.
+	     * Save the inode and device numbers.  Clear the verify flag.
 	     */
 		c->cd.inode = sb.st_ino;
-		(void) strcpy(c->cd.name, path);
 		c->cd.rdev = sb.st_rdev;
 		c->cd.v = 0;
 	    /*
@@ -303,7 +303,7 @@ read_clone()
 		cn++;
 		if (strcmp(cn, "icmp") == 0 || strcmp(cn, "ip")  == 0
 		||  strcmp(cn, "tcp")  == 0 || strcmp(cn, "udp") == 0)
-		    c->n = cn - path;
+		    c->n = cn - fp;
 		else
 		    c->n = 0;
 	    /*
@@ -323,16 +323,8 @@ read_clone()
 	     */
 		if (!(p = (struct pseudo *) malloc(sizeof(struct pseudo)))) {
 		    (void) fprintf(stderr,
-			"%s: no space for pseudo device: %s\n",
-			Pn, path);
-		    Exit(1);
-		}
-	    /*
-	     * Allocate space for the path name.
-	     */
-		if (!(p->pd.name = (char *)malloc(nl))) {
-		    (void) fprintf(stderr,
-			"%s: no space for pseudo name: %s\n", Pn, path);
+			"%s: no space for pseudo device: ", Pn);
+		    safestrprt(fp, stderr, 1);
 		    Exit(1);
 		}
 	    /*
@@ -340,7 +332,8 @@ read_clone()
 	     * verify flag.  Link the entry to the pseudo chain.
 	     */
 		p->pd.inode = sb.st_ino;
-		(void) strcpy(p->pd.name, path);
+		p->pd.name = fp;
+		fp = (char *)NULL;
 		p->pd.rdev = sb.st_rdev;
 		p->pd.v = 0;
 		p->next = Pseudo;
@@ -348,6 +341,10 @@ read_clone()
 	    }
 	}
 	(void) closedir(dfp);
+	if (fp)
+	    (void) free((FREE_P *)fp);
+	if (path)
+	    (void) free((FREE_P *)path);
 }
 #endif	/* solaris */
 
@@ -363,32 +360,30 @@ readdev(skip)
 {
 
 #if	defined(HASDCACHE)
-	int dcrd;
+	int dcrd = 0;
 #endif	/* defined(HASDCACHE) */
 
-	struct clone *c;
 	DIR *dfp;
 	int dn = 0;
 	struct dirent *dp;
 	char **dstk = (char **)NULL;
 	int dx = 0;
+	char *fp = (char *)NULL;
+	MALLOC_S fpl;
 	int i;
 
 #if	defined(HASBLKDEV)
 	int j = 0;
 #endif	/* defined(HASBLKDEV) */
 
-	MALLOC_S nl;
-	char path[MAXNAMLEN+1];
-	int pl;
+	char *path = (char *)NULL;
+	char *ppath = (char *)NULL;
+	MALLOC_S pl;
 	struct stat sb;
 
-#if	defined(solaris)
-	char *cn;
-	STRNCPY_L dnamel;
-	struct pseudo *p;
-	char ppath[MAXPATHLEN+1];
-#endif	/* defined(solaris) */
+#if	!defined(solaris)
+	struct clone *c;
+#endif	/* !defined(solaris) */
 
 	if (Sdev)
 	    return;
@@ -409,7 +404,13 @@ readdev(skip)
 	Devx = 0;
 
 #if	defined(solaris)
-	(void) sprintf(ppath, "%s/pseudo", DVCH_DEVPATH);
+	if (!(ppath = mkstrcat(DVCH_DEVPATH, -1, "/", 1, "pseudo", -1,
+		      (MALLOC_S *)NULL)))
+	{
+	    (void) fprintf(stderr, "%s: no space for: %s/pseudo\n",
+		Pn, DVCH_DEVPATH);
+	    Exit(1);
+	}
 	read_clone();
 #endif	/* defined(solaris) */
 
@@ -418,19 +419,30 @@ readdev(skip)
  * Unstack the next directory.
  */
 	while (--dx >= 0) {
-	    (void) strcpy(path, dstk[dx]);
-	    if (!(dfp = opendir(path))) {
+	    if (!(dfp = opendir(dstk[dx]))) {
 
 #if	defined(WARNDEVACCESS)
-		if (!Fwarn)
-		    (void) fprintf(stderr, "%s: WARNING: can't open %s\n",
-		    Pn, path);
+		if (!Fwarn) {
+		    (void) fprintf(stderr, "%s: WARNING: can't open: ", Pn);
+		    safestrprt(dstk[dx], stderr, 1);
+		}
 #endif	/* defined(WARNDEVACCESS) */
 
 		continue;
 	    }
-	    (void) strcat(path, "/");
-	    pl = strlen(path);
+	/*
+	 * Create a directory name buffer with a trailing slash.
+	 */
+	    if (path) {
+		(void) free((FREE_P *)NULL);
+		path = (char *)NULL;
+	    }
+	    if (!(path = mkstrcat(dstk[dx], -1, "/", 1, (char *)NULL, -1, &pl)))
+	    {
+		(void) fprintf(stderr, "%s: no space for: ", Pn);
+		safestrprt(dstk[dx], stderr, 1);
+		Exit(1);
+	    }
 	    (void) free((FREE_P *)dstk[dx]);
 	    dstk[dx] = (char *)NULL;
 	/*
@@ -442,32 +454,23 @@ readdev(skip)
 	    /*
 	     * Form the full path name and get its status.
 	     */
-
-#if	defined(solaris)
-		dnamel = strlen(dp->d_name);
-		if ((nl = pl + dnamel) >= sizeof(path))
-#else	/* !defined(solaris) */
-		if ((nl = pl + dp->d_namlen) >= sizeof(path))
-#endif	/* solaris */
-
+		if (fp) {
+		    (void) free((FREE_P *)fp);
+		    fp = (char *)NULL;
+		}
+		if (!(fp = mkstrcat(path, pl, dp->d_name, -1, (char *)NULL, -1,
+			   &fpl)))
 		{
-		    (void) fprintf(stderr, "%s: /dev entry name too long: %s\n",
-			Pn, dp->d_name);
+		    (void) fprintf(stderr, "%s: no space for: ", Pn);
+		    safestrprt(path, stderr, 0);
+		    safestrprt(dp->d_name, stderr, 1);
 		    Exit(1);
 		}
 
-#if	defined(solaris)
-		(void) strncpy(&path[pl], dp->d_name, dnamel);
-#else
-		(void) strncpy(&path[pl], dp->d_name, (STRNCPY_L)dp->d_namlen);
-#endif	/* solaris */
-
-		path[nl++] = '\0';
-
 #if	defined(USE_STAT)
-		if (stat(path, &sb) != 0)
+		if (stat(fp, &sb) != 0)
 #else	/* !defined(USE_STAT) */
-		if (lstat(path, &sb) != 0)
+		if (lstat(fp, &sb) != 0)
 #endif	/* defined(USE_STAT) */
 
 		{
@@ -475,9 +478,13 @@ readdev(skip)
 			continue;
 
 #if	defined(WARNDEVACCESS)
-		    if (!Fwarn)
-			(void) fprintf(stderr, "%s: can't stat %s: %s\n",
-			    Pn, path, strerror(errno));
+		    if (!Fwarn) {
+			int errno_save = errno;
+
+			(void) fprintf(stderr, "%s: can't stat ", Pn);
+			safestrprt(fp, stderr, 0);
+			(void) fprintf(stderr, ": %s\n", strerror(errno_save));
+		    }
 #endif	/* defined(WARNDEVACCESS) */
 
 		    continue;
@@ -492,11 +499,11 @@ readdev(skip)
 		 * Skip Solaris /DVCH_DEV_PATH/pseudo sub-directory;
 		 * it has been examined in read_clone().
 		 */
-		    if (strcmp(path, ppath) == 0)
+		    if (strcmp(fp, ppath) == 0)
 			continue;
 #endif	/* solaris */
 
-		    (void) stkdir(&dstk, &dn, &dx, path);
+		    (void) stkdir(&dstk, &dn, &dx, fp);
 			continue;
 		}
 		if ((sb.st_mode & S_IFMT) == S_IFCHR) {
@@ -504,7 +511,7 @@ readdev(skip)
 		/*
 		 * Make Devtp[] entry.
 		 */
-		    make_devtp(&sb, path, nl);
+		    make_devtp(&sb, fp, fpl);
 
 #if	!defined(solaris)
 		/*
@@ -525,16 +532,16 @@ readdev(skip)
 		    /*
 		     * Allocate space for the path name.
 		     */
-			if (!(c->cd.name = (char *)malloc(nl))) {
+			if (!(c->cd.name = mkstrcpy(fp, (MALLOC_S *)NULL))) {
 			    (void) fprintf(stderr,
-				"%s: no space for clone name: %s\n", Pn, path);
+				"%s: no space for clone name: ", Pn);
+			    safestrprt(fp, stderr, 1);
 			    Exit(1);
 			}
 		    /*
 		     * Fill in the clone entry and link it to the rest.
 		     */
 			c->cd.inode = sb.st_ino;
-			(void) strcpy(c->cd.name, path);
 			c->n = 0;
 			c->cd.rdev = sb.st_rdev;
 			c->cd.v = 0;
@@ -567,12 +574,8 @@ readdev(skip)
 		    }
 		    BDevtp[j].rdev = sb.st_rdev;
 		    BDevtp[j].inode = sb.st_ino;
-		    if (!(BDevtp[j].name = (char *)malloc(nl))) {
-			(void) fprintf(stderr,
-			    "%s: no space for /dev/%s\n", Pn, dp->d_name);
-			Exit(1);
-		    }
-		    (void) strcpy(BDevtp[j].name, path);
+		    BDevtp[j].name = fp;
+		    fp = (char *)NULL;
 		    BDevtp[j].v = 0;
 		    j++;
 		}
@@ -582,10 +585,16 @@ readdev(skip)
 	    (void) closedir(dfp);
 	}
 /*
- * Free any directory stack space.
+ * Free any allocated space.
  */
-	if (!dstk)
+	if (dstk)
 	    (void) free((FREE_P *)dstk);
+	if (fp)
+	    (void) free((FREE_P *)fp);
+	if (path)
+	    (void) free((FREE_P *)path);
+	if (ppath)
+	    (void) free((FREE_P *)ppath);
 /*
  * Reduce the BDevtp[] (optional) and Devtp[] tables to their minimum
  * sizes; allocate and build sort pointer lists; and sort the tables by
@@ -600,10 +609,10 @@ readdev(skip)
 			 (MALLOC_S)(sizeof(struct l_dev) * BNdev));
 	    }
 	    if (!(BSdev = (struct l_dev **)malloc(
-		(MALLOC_S)(sizeof(struct l_dev *) * BNdev))))
+			  (MALLOC_S)(sizeof(struct l_dev *) * BNdev))))
 	    {
 		(void) fprintf(stderr,
-		    "%s: no space for block device pointers\n", Pn);
+		    "%s: no space for block device sort pointers\n", Pn);
 		Exit(1);
 	    }
 	    for (j = 0; j < BNdev; j++) {
@@ -626,10 +635,10 @@ readdev(skip)
 			(MALLOC_S)(sizeof(struct l_dev) * Ndev));
 	    }
 	    if (!(Sdev = (struct l_dev **)malloc(
-		(MALLOC_S)(sizeof(struct l_dev *) * Ndev))))
+			 (MALLOC_S)(sizeof(struct l_dev *) * Ndev))))
 	    {
 		(void) fprintf(stderr,
-		    "%s: no space for character device pointers\n", Pn);
+		    "%s: no space for character device sort pointers\n", Pn);
 		Exit(1);
 	    }
 	    for (i = 0; i < Ndev; i++) {
@@ -664,13 +673,9 @@ readdev(skip)
 void
 clr_sect()
 {
-	struct clone *c, *c1;
-
-#if	defined(solaris)
-	struct pseudo *p, *p1;
-#endif	defined(solaris)
-
 	if (Clone) {
+	    struct clone *c, *c1;
+
 	    for (c = Clone; c; c = c1) {
 		c1 = c->next;
 		if (c->cd.name)
@@ -682,6 +687,8 @@ clr_sect()
 
 # if	defined(solaris)
 	if (Pseudo) {
+	    struct pseudo *p, *p1;
+
 	    for (p = Pseudo; p; p = p1) {
 		p1 = p->next;
 		if (p->pd.name)
@@ -716,10 +723,12 @@ rw_clone_sect(m)
 
 bad_clone_sect:
 
-		if (!Fwarn)
+		if (!Fwarn) {
 		    (void) fprintf(stderr,
-			"%s: bad clone section header in %s: %s",
-			Pn, DCpath[DCpathX], buf);
+			"%s: bad clone section header in %s: ",
+			Pn, DCpath[DCpathX]);
+		    safestrprt(buf, stderr, 1);
+		}
 		return(1);
 	    }
 	    (void) crc(buf, strlen(buf), &DCcksum);
@@ -733,10 +742,11 @@ bad_clone_sect:
 	 */
 	    for (i = 0; i < n; i++) {
 		if (!fgets(buf, sizeof(buf), DCfs)) {
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad clone line in %s: %s",
-			    Pn, DCpath[DCpathX], buf);
+			    "%s: bad clone line in %s: ", Pn, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
 		(void) crc(buf, strlen(buf), &DCcksum);
@@ -745,7 +755,8 @@ bad_clone_sect:
 	     */
 		if (!(c = (struct clone *)calloc(1, sizeof(struct clone)))) {
 		    (void) fprintf(stderr,
-			"%s: no space for cached clone: %s", Pn, buf);
+			"%s: no space for cached clone: ", Pn);
+		    safestrprt(buf, stderr, 1);
 		    Exit(1);
 		}
 	    /*
@@ -758,9 +769,11 @@ bad_clone_sect:
 		if (buf[0] != ' '
 		||  !(cp = x2dev(&buf[1], &c->cd.rdev)) || *cp++ != ' ')
 		{
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad cached clone device: %s", Pn, buf);
+			    "%s: bad cached clone device: ", Pn);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
 	    /*
@@ -768,10 +781,11 @@ bad_clone_sect:
 	     */
 		for (c->n = 0; *cp != ' '; cp++) {
 		    if (*cp < '0' || *cp > '9') {
-			if (!Fwarn)
+			if (!Fwarn) {
 			    (void) fprintf(stderr,
-				"%s: bad cached clone network flag: %s",
-				Pn, buf);
+				"%s: bad cached clone network flag: ", Pn);
+			    safestrprt(buf, stderr, 1);
+			}
 			return(1);
 		    }
 		    c->n = (c->n * 10) + (int)(*cp - '0');
@@ -781,10 +795,11 @@ bad_clone_sect:
 	     */
 		for (c->cd.inode = (ino_t)0, ++cp; *cp != ' '; cp++) {
 		    if (*cp < '0' || *cp > '9') {
-			if (!Fwarn)
+			if (!Fwarn) {
 			    (void) fprintf(stderr,
-				"%s: bad cached clone inode number: %s",
-				Pn, buf);
+				"%s: bad cached clone inode number: ", Pn);
+			    safestrprt(buf, stderr, 1);
+			}
 			return(1);
 		    }
 		    c->cd.inode = (c->cd.inode * 10) + (ino_t)(*cp - '0');
@@ -793,18 +808,20 @@ bad_clone_sect:
 	     * Enter the clone path name.
 	     */
 		if ((len = strlen(++cp)) < 2 || *(cp + len - 1) != '\n') {
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad cached clone path: %s", Pn, buf);
+			    "%s: bad cached clone path: ", Pn);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
-		if (!(c->cd.name = (char *)malloc(len))) {
+		*(cp + len - 1) = '\0';
+		if (!(c->cd.name = mkstrcpy(cp, (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr,
-			"%s: no space for cached clone path: %s", Pn, buf);
+			"%s: no space for cached clone path: ", Pn);
+		    safestrprt(buf, stderr, 1);
 		    Exit(1);
 		}
-		*(cp + len - 1) = '\0';
-		(void) strcpy(c->cd.name, cp);
 		c->cd.v = 0;
 		c->next = Clone;
 		Clone = c;
@@ -829,7 +846,7 @@ bad_clone_sect:
 	 * not use the new format lines.
 	 */
 	    for (c = Clone; c; c = c->next) {
-		(void) sprintf(buf, " %x %d %ld %s\n", c->cd.rdev, c->n,
+		(void) sprintf(buf, " %x %d %ld %s\n", (int)c->cd.rdev, c->n,
 		    c->cd.inode, c->cd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
@@ -885,10 +902,12 @@ rw_pseudo_sect(m)
 
 bad_pseudo_sect:
 
-		if (!Fwarn)
+		if (!Fwarn) {
 		    (void) fprintf(stderr,
-			"%s: bad pseudo section header in %s: %s",
-			Pn, DCpath[DCpathX], buf);
+			"%s: bad pseudo section header in %s: ",
+			Pn, DCpath[DCpathX]);
+		    safestrprt(buf, stderr, 1);
+		}
 		return(1);
 	    }
 	    (void) crc(buf, strlen(buf), &DCcksum);
@@ -902,12 +921,11 @@ bad_pseudo_sect:
 	 */
 	    for (i = 0; i < n; i++) {
 		if (!fgets(buf, sizeof(buf), DCfs)) {
-
-bad_pseudo_line:
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad pseudo line in %s: %s",
-			    Pn, DCpath[DCpathX], buf);
+			    "%s: bad pseudo line in %s: ", Pn, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
 		(void) crc(buf, strlen(buf), &DCcksum);
@@ -916,7 +934,8 @@ bad_pseudo_line:
 	     */
 		if (!(p = (struct pseudo *)calloc(1, sizeof(struct pseudo)))) {
 		    (void) fprintf(stderr,
-			"%s: no space for cached pseudo: %s", Pn, buf);
+			"%s: no space for cached pseudo: ", Pn);
+		    safestrprt(buf, stderr, 1);
 		    Exit(1);
 		}
 	    /*
@@ -929,9 +948,11 @@ bad_pseudo_line:
 		if (buf[0] != ' '
 		||  !(cp = x2dev(&buf[1], &p->pd.rdev)) || *cp++ != ' ')
 		{
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad cached pseudo device: %s", Pn, buf);
+			    "%s: bad cached pseudo device: ", Pn);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
 	    /*
@@ -939,10 +960,11 @@ bad_pseudo_line:
 	     */
 		for (p->pd.inode = (ino_t)0; *cp != ' '; cp++) {
 		    if (*cp < '0' || *cp > '9') {
-			if (!Fwarn)
+			if (!Fwarn) {
 			    (void) fprintf(stderr,
-				"%s: bad cached clone inode number: %s",
-				Pn, buf);
+				"%s: bad cached clone inode number: ", Pn);
+			    safestrprt(buf, stderr, 1);
+			}
 			return(1);
 		    }
 		    p->pd.inode = (p->pd.inode * 10) + (ino_t)(*cp - '0');
@@ -956,14 +978,17 @@ bad_pseudo_line:
 	     * not use the new format lines.
 	     */
 		if ((len = strlen(++cp)) < 2 || *(cp + len - 1) != '\n') {
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: bad cached pseudo path: %s", Pn, buf);
+			    "%s: bad cached pseudo path: ", Pn);
+			safestrprt(buf, stderr, 1);
+		    }
 		    return(1);
 		}
 		if (!(p->pd.name = (char *)malloc(len))) {
 		    (void) fprintf(stderr,
-			"%s: no space for cached pseudo path: %s", Pn, buf);
+			"%s: no space for cached pseudo path: ", Pn);
+		    safestrprt(buf, stderr, 1);
 		    Exit(1);
 		}
 		*(cp + len - 1) = '\0';
@@ -992,8 +1017,8 @@ bad_pseudo_line:
 	 * not use the new format lines.
 	 */
 	    for (p = Pseudo; p; p = p->next) {
-		(void) sprintf(buf, " %x %ld %s\n", p->pd.rdev, p->pd.inode,
-		    p->pd.name);
+		(void) sprintf(buf, " %x %ld %s\n", (int)p->pd.rdev,
+		    p->pd.inode, p->pd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
 	    }
@@ -1120,7 +1145,7 @@ rmdupdev(dp, n, ty)
 	if (n == j)
 	    return(n);
 	if (!(*dp = (struct l_dev **)realloc((MALLOC_P *)*dp,
-	    (MALLOC_S)(j * sizeof(struct l_dev *)))))
+		    (MALLOC_S)(j * sizeof(struct l_dev *)))))
 	{
 	    (void) fprintf(stderr, "%s: can't realloc %s device pointers\n",
 		Pn, ty ? "char" : "block");
@@ -1141,40 +1166,31 @@ stkdir(d, n, x, p)
 	int *x;			/* current index */
 	char *p;		/* directory path */
 {
-	if (!*d) {
-
-	/*
-	 * Allocate first entry.
-	 */
-	    if (!(*d = (char **)malloc(sizeof(char *)))) {
-
-stkdir_nospace:
-
+	MALLOC_S len;
+/*
+ * Provide adequate directory stack space.
+ */
+	if (*x >= *n) {
+	    *n += 10;
+	    len = (MALLOC_S)(*n * sizeof(char *));
+	    if (!*d)
+		*d = (char **)malloc(len);
+	    else
+		*d = (char **)realloc((MALLOC_P *)*d, len);
+	    if (!*d) {
 		(void) fprintf(stderr,
-		    "%s: no space for directory stack at %s\n", Pn, p);
+		    "%s: no space for directory stack for: ", Pn);
+		safestrprt(p, stderr, 1);
 		Exit(1);
 	    }
-	    *n = 1;
-	    *x = 0;
-	} else if (*x >= *n) {
-
-	/*
-	 * Allocate additional space as required.
-	 */
-	    *n += 1;
-	    if (!(*d = (char **)realloc((MALLOC_P *)*d,
-		       (MALLOC_S)(*n * sizeof(char *)))))
-	    {
-		goto stkdir_nospace;
-	    }
 	}
 /*
  * Allocate space for the name, copy it there and put its pointer on the stack.
  */
-	if (!((*d)[*x] = (char *)malloc((MALLOC_S)(strlen(p) + 1)))) {
-	    (void) fprintf(stderr, "%s: no space for %s\n", Pn, p);
+	if (!((*d)[*x] = mkstrcpy(p, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no space for: ", Pn);
+	    safestrprt(p, stderr, 1);
 	    Exit(1);
 	}
-	(void) strcpy((*d)[*x], p);
 	*x += 1;
 }
