@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.1 97/02/24 07:47:17 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.2 97/10/24 07:58:17 abe Exp Locker: abe $";
 #endif
 
 
@@ -53,14 +53,14 @@ char *AFSApath = (char *)NULL;		/* alternate AFS name list path
 #if	defined(solaris)
 int AFSfstype = -1;			/* AFS file system type index */
 # endif	/* defined(solaris) */
-struct vfs *AFSVfsp = (struct vfs *)NULL;
-					/* AFS vfs struct kernel address */
+
+KA_T AFSVfsp = (KA_T)NULL;		/* AFS vfs struct kernel address */
 #endif	/* defined(HAS_AFS) */
 
 struct clone *Clone = NULL;		/* clone list */
 
 #if	defined(solaris)
-int CloneMaj;				/* clone major device number */
+major_t CloneMaj;			/* clone major device number */
 #endif	/* defined(solaris) */
 
 #if	!defined(solaris)
