@@ -1,5 +1,5 @@
 /*
- * rnam.c -- Sun format name cache functions for lsof library
+ * rnch.c -- Sun format name cache functions for lsof library
  */
 
 
@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnch.c,v 1.4 98/01/16 20:02:48 abe Exp $";
+static char *rcsid = "$Id: rnch.c,v 1.5 98/03/06 08:41:03 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -65,6 +65,10 @@ static char *rcsid = "$Id: rnch.c,v 1.4 98/01/16 20:02:48 abe Exp $";
  *	Define NCACHE_NXT if the kernel's name cache is a linked list, starting
  *	at the X_NCACHE address, rather than a table, starting at that address.
  *
+ *	Define any of the following casts that differ from their defaults:
+ *
+ *		NCACHE_SZ_CAST	cast for X_NCACHE (default int)
+ *
  * The caller may:
  *
  *	Define NCACHE_DP	as the name of the element in the
@@ -115,12 +119,17 @@ static char *rcsid = "$Id: rnch.c,v 1.4 98/01/16 20:02:48 abe Exp $";
  */
 
 static int Mch;				/* name cache hash mask */
-static int Nc = 0;			/* size of name cache */
+
+# if	!defined(NCACHE_NC_CAST)
+#define	NCACHE_SZ_CAST	int
+# endif	/* !defined(NCACHE_NC_CAST) */
+
+static NCACHE_SZ_CAST Nc = 0;		/* size of name cache */
 static int Nch = 0;			/* size of name cache hash pointer
 					 * table */
 struct l_nch {
-	struct vnode *vp;		/* vnode address */
-	struct vnode *dp;		/* parent vnode address */
+	KA_T vp;			/* vnode address */
+	KA_T dp;			/* parent vnode address */
 	struct l_nch *pa;		/* parent Ncache address */
 
 # if	defined(NCACHE_NODEID)
@@ -139,14 +148,14 @@ static struct l_nch **Nchash = (struct l_nch **)NULL;
 static int Ncfirst = 1;			/* first-call status */
 
 # if	defined(NCACHE_NODEID)
-_PROTOTYPE(static struct l_nch *ncache_addr,(unsigned long i, struct vnode *v));
+_PROTOTYPE(static struct l_nch *ncache_addr,(unsigned long i, KA_T v));
 #define ncachehash(i,v)		Nchash+(((((int)(v)>>2)+((int)(i)))*31415)&Mch)
 # else	/* !defined(NCACHE_NODEID) */
-_PROTOTYPE(static struct l_nch *ncache_addr,(struct vnode *v));
+_PROTOTYPE(static struct l_nch *ncache_addr,(KA_T v));
 #define ncachehash(v)		Nchash+((((int)(v)>>2)*31415)&Mch)
 # endif	/* defined(NCACHE_NODEID) */
 
-_PROTOTYPE(static int ncache_isroot,(struct vnode *va, char *cp));
+_PROTOTYPE(static int ncache_isroot,(KA_T va, char *cp));
 
 #define DEFNCACHESZ	1024	/* local size if X_NCSIZE kernel value < 1 */
 #define	LNCHINCRSZ	64	/* local size increment */
@@ -184,7 +193,7 @@ ncache_addr(v)
 	unsigned long i;			/* capability ID */
 # endif	/* defined(NCACHE_NODEID) */
 
-	struct vnode *v;			/* vnode's address */
+	KA_T v;					/* vnode's address */
 {
 	struct l_nch **hp;
 
@@ -214,8 +223,8 @@ ncache_addr(v)
 
 static int
 ncache_isroot(va, cp)
-	struct vnode *va;			/* kernel vnode address */
-	char *cp;				/* partial path */
+	KA_T va;			/* kernel vnode address */
+	char *cp;			/* partial path */
 {
 	char buf[MAXPATHLEN];
 	int i;
@@ -225,7 +234,7 @@ ncache_isroot(va, cp)
 	struct vnode v;
 	static int vca = 0;
 	static int vcn = 0;
-	static struct vnode **vc = (struct vnode **)NULL;
+	static KA_T *vc = (KA_T *)NULL;
 
 	if (!va)
 	    return(0);
@@ -277,11 +286,11 @@ ncache_isroot(va, cp)
  */
 	if (vcn >= vca) {
 	    vca += 10;
-	    len = (MALLOC_S)(vca * sizeof(struct vnode *));
+	    len = (MALLOC_S)(vca * sizeof(KA_T));
 	    if (!vc)
-		vc = (struct vnode **)malloc(len);
+		vc = (KA_T *)malloc(len);
 	    else
-		vc = (struct vnode **)realloc(vc, len);
+		vc = (KA_T *)realloc(vc, len);
 	    if (!vc) {
 		(void) fprintf(stderr, "%s: no space for root vnode table\n",
 		    Pn);
@@ -306,7 +315,7 @@ ncache_load()
 	static int iNc = 0;
 	struct ncache *kc;
 	static KA_T kp = (KA_T)NULL;
-	unsigned long v;
+	KA_T v;
 
 # if	defined(HASDNLCPTR)
 	static int na = 0;
@@ -333,14 +342,15 @@ ncache_load()
 	 */
 
 # if	defined(X_NCSIZE)
-	    v = (unsigned long)0;
+	    v = (KA_T)0;
 	    if (get_Nl_value(X_NCSIZE, (struct drive_Nl *)NULL, &v) < 0
 	    ||  !v
 	    ||  kread((KA_T)v, (char *)&Nc, sizeof(Nc)))
 	    {
 		if (!Fwarn)
 		(void) fprintf(stderr,
-		    "%s: WARNING: can't read name cache size: %#lx\n", Pn, v);
+		    "%s: WARNING: can't read name cache size: %s\n",
+			Pn, print_kptr(v, (char *)NULL));
 		iNc = Nc = 0;
 		return;
 	    }
@@ -363,8 +373,8 @@ ncache_load()
 	 */
 
 # if	defined(ADDR_NCACHE)
-	    if (get_Nl_value(X_NCACHE, (struct drive_Nl *)NULL,
-			     (unsigned long *)&kp) < 0
+	    kp = (KA_T)0;
+	    if (get_Nl_value(X_NCACHE,(struct drive_Nl *)NULL,(KA_T *)&kp) < 0
 	    || !kp) {
 		if (!Fwarn)
 		    (void) fprintf(stderr,
@@ -373,14 +383,14 @@ ncache_load()
 		return;
 	    }
 # else	/* !defined(ADDR_NCACHE) */
-	    v = (unsigned long)0;
+	    v = (KA_T)0;
 	    if (get_Nl_value(X_NCACHE, (struct drive_Nl *)NULL, &v) < 0
 	    || !v
 	    ||  kread((KA_T)v, (char *)&kp, sizeof(kp))) {
 		if (!Fwarn)
 		    (void) fprintf(stderr,
-			"%s: WARNING: can't read name cache ptr: %#lx\n",
-			Pn, v);
+			"%s: WARNING: can't read name cache ptr: %s\n",
+			Pn, print_kptr(v, (char *)NULL));
 		iNc = Nc = 0;
 		return;
 	    }
@@ -452,8 +462,8 @@ no_local_space:
 	if (kread(kp, (char *)kca, (Nc * sizeof(struct ncache)))) {
 	    if (!Fwarn)
 		(void) fprintf(stderr,
-		    "%s: WARNING: can't read kernel's name cache: %#lx\n",
-		    Pn, kp);
+		    "%s: WARNING: can't read kernel's name cache: %s\n",
+		    Pn, print_kptr(kp, (char *)NULL));
 	    Nc = 0;
 	    return;
 	}
@@ -550,8 +560,8 @@ no_local_space:
 	/*
 	 * Complete the local cache entry.
 	 */
-	    lc->vp = kc->NCACHE_VP;
-	    lc->dp = kc->NCACHE_DP;
+	    lc->vp = (KA_T)kc->NCACHE_VP;
+	    lc->dp = (KA_T)kc->NCACHE_DP;
 	    lc->pa = (struct l_nch *)NULL;
 	    lc->nm = cp;
 	    lc->nl = len;
@@ -705,9 +715,9 @@ ncache_lookup(buf, blen, fp)
 	if (!Nc
 
 # if	defined(NCACHE_NODEID)
-	||  !(lc = ncache_addr(Lf->id, (struct vnode *)Lf->na))
+	||  !(lc = ncache_addr(Lf->id, Lf->na))
 # else	/* !defined(NCACHE_NODEID) */
-	||  !(lc = ncache_addr((struct vnode *)Lf->na))
+	||  !(lc = ncache_addr(Lf->na))
 # endif	/* defined(NCACHE_NODEID) */
 
 	) {
