@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.13 98/02/16 15:54:56 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.16 98/03/06 07:55:00 abe Exp $";
 #endif
 
 
@@ -65,13 +65,15 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	int c, i, n, rv;
+	int c, ev, i, n, rv;
 	char *cp;
 	int err = 0;
 	int fh = 0;
+	MALLOC_S len;
 	struct lfile *lf;
 	struct nwad *np, *npn;
 	char options[128];
+	int rc = 0;
 	struct sfile *sfp;
 	struct lproc **slp = (struct lproc **)NULL;
 	int sp = 0;
@@ -79,481 +81,476 @@ main(argc, argv)
 	int version = 0;
 
 	if ((Pn = strrchr(argv[0], '/')))
-		Pn++;
+	    Pn++;
 	else
-		Pn = argv[0];
+	    Pn = argv[0];
 /*
  * Common initialization.
  */
 	Mypid = getpid();
 	if ((Mygid = (gid_t)getgid()) != getegid())
-		Setgid = 1;
+	    Setgid = 1;
 	if ((Myuid = (uid_t)getuid()) && !geteuid())
-		Setuidroot = 1;
-	if (!(Namech = (char *)malloc(MAXPATHLEN))) {
-		(void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
-		Exit(1);
+	    Setuidroot = 1;
+	if (!(Namech = (char *)malloc(MAXPATHLEN + 1))) {
+	    (void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
+	    Exit(1);
 	}
 /*
  * Create option mask.
  */
 	(void) sprintf(options,
-		"?a%sbc:%sd:%sfF:g:hi:%sl%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
+	    "?a%sbc:%sd:%sfF:g:hi:%sl%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
-		"A:",
+	    "A:",
 #else	/* !defined(HAS_AFS) || !defined(HASAOPT) */
-		"",
+	    "",
 #endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
 #if	defined(HASNCACHE)
-		"C",
+	    "C",
 #else	/* !defined(HASNCACHE) */
-		"",
+	    "",
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASDCACHE)
-		"D:",
+	    "D:",
 #else	/* !defined(HASDCACHE) */
-		"",
+	    "",
 #endif	/* defined(HASDCACHE) */
 
 #if	defined(HASKOPT)
-		"k:",
+	    "k:",
 #else	/* !defined(HASKOPT) */
-		"",
+	    "",
 #endif	/* defined(HASKOPT) */
 
 #if	defined(HASMOPT)
-		"m:",
+	    "m:",
 #else	/* !defined(HASMOPT) */
-		"",
+	    "",
 #endif	/* defined(HASMOPT) */
 
 #if	defined(HASPPID)
-		"R",
+	    "R",
 #else	/* !defined(HASPPID) */
-		"",
+	    "",
 #endif	/* defined(HASPPID) */
 
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
-		(Myuid == 0) ? "X" : ""
+	    (Myuid == 0) ? "X" : ""
 # else	/* !defined(HASXOPT_ROOT) */
-		"X"
+	    "X"
 # endif	/* defined(HASXOPT_ROOT) */
 #else	/* !defined(HASXOPT) */
-		""
+	    ""
 #endif	/* defined(HASXOPT) */
 
-		);
+	    );
 /*
  * Loop through options.
  */
 	while ((c = GetOpt(argc, argv, options, &rv)) != EOF) {
-		if (rv) {
-			err = 1;
-			continue;
-		}
-		switch(c) {
-
-		case 'a':
-			Fand = 1;
-			break;
+	    if (rv) {
+		err = 1;
+		continue;
+	    }
+	    switch(c) {
+	    case 'a':
+		Fand = 1;
+		break;
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
-		case 'A':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    (void) fprintf(stderr,
-				"%s: -A not followed by path\n", Pn);
-			    err = 1;
-			} else
-			    AFSApath = GOv;
-			break;
+	    case 'A':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    (void) fprintf(stderr, "%s: -A not followed by path\n", Pn);
+		    err = 1;
+		} else
+		    AFSApath = GOv;
+		break;
 #endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
-		case 'b':
-			Fblock = 1;
-			break;
-		case 'c':
-			if (enter_str_lst("c", GOv, &Cmdl))
-			    err = 1;
-			break;
+	    case 'b':
+		Fblock = 1;
+		break;
+	    case 'c':
+		if (enter_str_lst("c", GOv, &Cmdl))
+		    err = 1;
+		break;
 
 #if	defined(HASNCACHE)
-		case 'C':
-			Fncache = (GOp == '-') ? 0 : 1;
-			break;
+	    case 'C':
+		Fncache = (GOp == '-') ? 0 : 1;
+		break;
 #endif	/* defined(HASNCACHE) */
 
-		case 'd':
-			if (enter_fd(GOv))
-			    err = 1;
-			break;
+	    case 'd':
+		if (enter_fd(GOv))
+		    err = 1;
+		break;
 
 #if	defined(HASDCACHE)
-		case 'D':
-			if (ctrl_dcache(GOv))
-			    err = 1;
-			break;
+	    case 'D':
+		if (ctrl_dcache(GOv))
+		    err = 1;
+		break;
 #endif	/* defined(HASDCACHE) */
 
-		case 'f':
-			Ffilesys = (GOp == '+') ? 2 : 1;
-			break;
-		case 'F':
-			if (!GOv || *GOv == '-' || *GOv == '+'
-			||  strcmp(GOv, "0") == 0) {
-			    if (GOv) {
-				if (*GOv == '-' || *GOv == '+') {
-				    GOx1 = GObk[0];
-				    GOx2 = GObk[1];
-				} else if (*GOv == '0')
-				    Terminator = '\0';
-			    }
-			    for (i = 0; FieldSel[i].nm; i++) {
+	    case 'f':
+		Ffilesys = (GOp == '+') ? 2 : 1;
+		break;
+	    case 'F':
+		if (!GOv || *GOv == '-' || *GOv == '+'
+		||  strcmp(GOv, "0") == 0) {
+		    if (GOv) {
+			if (*GOv == '-' || *GOv == '+') {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1];
+			} else if (*GOv == '0')
+			    Terminator = '\0';
+		    }
+		    for (i = 0; FieldSel[i].nm; i++) {
 
 #if	!defined(HASPPID)
-				if (FieldSel[i].id == LSOF_FID_PPID)
-				    continue;
+			if (FieldSel[i].id == LSOF_FID_PPID)
+			    continue;
 #endif	/* !defined(HASPPID) */
 
-				FieldSel[i].st = 1;
-			    }
-			    Ffield = 1;
-			    break;
-			}
-			if (strcmp(GOv, "?") == 0) {
-			    fh = 1;
-			    break;
-			}
-			for (; *GOv; GOv++) {
-			    for (i = 0; FieldSel[i].nm; i++) {
+			FieldSel[i].st = 1;
+		    }
+		    Ffield = 1;
+		    break;
+		}
+		if (strcmp(GOv, "?") == 0) {
+		    fh = 1;
+		    break;
+		}
+		for (; *GOv; GOv++) {
+		    for (i = 0; FieldSel[i].nm; i++) {
 
 #if	!defined(HASPPID)
-				if (FieldSel[i].id == LSOF_FID_PPID)
-				    continue;
+			if (FieldSel[i].id == LSOF_FID_PPID)
+			    continue;
 #endif	/* !defined(HASPPID) */
 
-				if (FieldSel[i].id == *GOv) {
-				    FieldSel[i].st = 1;
-				    if (i == LSOF_FIX_TERM)
-					Terminator = '\0';
-				    break;
-				}
-			    }
-			    if ( ! FieldSel[i].nm) {
-				(void) fprintf(stderr,
-				    "%s: unknown field: %c\n", Pn, *GOv);
-				err++;
-			    }
-			}
-			Ffield = 1;
-			break;
-		case 'g':
-			if (GOv) {
-			    if (*GOv == '-' || *GOv == '+') {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    } else if (enter_id(PGRP, GOv))
-				err = 1;
-			}
-			Fpgrp = 1;
-			break;
-		case 'h':
-		case '?':
-			Fhelp = 1;
-			break;
-		case 'i':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    Fnet = 1;
-			    if (GOv) {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    }
+			if (FieldSel[i].id == *GOv) {
+			    FieldSel[i].st = 1;
+			    if (i == LSOF_FIX_TERM)
+				Terminator = '\0';
 			    break;
 			}
-			if (enter_network_address(GOv))
-			    err = 1;
-			break;
-
-#if	defined(HASKOPT)
-		case 'k':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    (void) fprintf(stderr,
-				"%s: -k not followed by path\n", Pn);
-			    err = 1;
-			} else
-			    Nmlst = GOv;
-			break;
-#endif	/* defined(HASKOPT) */
-
-		case 'l':
-			Futol = 0;
-			break;
-
-#if	defined(HASMOPT)
-		case 'm':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    (void) fprintf(stderr,
-				"%s: -m not followed by path\n", Pn);
-			    err = 1;
-			} else
-			    Memory = GOv;
-			break;
-#endif	/* defined(HASMOPT) */
-
-		case 'M':
-			FportMap = (GOp == '+') ? 1 : 0;
-			break;
-		case 'n':
-			Fhost = (GOp == '-') ? 0 : 1;
-			break;
-		case 'N':
-			Fnfs = 1;
-			break;
-		case 'o':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    Foffset = 1;
-			    if (GOv) {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    }
-			    break;
-			}
-			for (cp = GOv, i = n = 0; *cp; cp++) {
-			    if (!isdigit((unsigned char)*cp))
-				break;
-			    i = (i * 10) + ((int)*cp - '0');
-			    n++;
-			}
-			if (n)
-			    OffDecDig = i;
-			else
-			    Foffset = 1;
-			if (*cp) {
-			    GOx1 = GObk[0];
-			    GOx2 = GObk[1] + n;
-			}
-			break;
-		case 'O':
-			Fovhd = (GOp == '-') ? 1 : 0;
-			break;
-		case 'p':
-			if (enter_id(PID, GOv))
-			    err = 1;
-			break;
-		case 'P':
-			Fport = (GOp == '-') ? 0 : 1;
-			break;
-		case 'r':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    if (GOv) {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    }
-			    RptTm = RPTTM;
-			    break;
-			}
-			for (cp = GOv, i = n = 0; *cp; cp++) {
-			    if (!isdigit((unsigned char)*cp))
-				break;
-			    i = (i * 10) + ((int)*cp - '0');
-			    n++;
-			}
-			if (n)
-			    RptTm = i;
-			else
-			    RptTm = RPTTM;
-			if (*cp) {
-			    GOx1 = GObk[0];
-			    GOx2 = GObk[1] + n;
-			}
-			break;
-
-#if	defined(HASPPID)
-		case 'R':
-			Fppid = 1;
-			break;
-#endif	/* defined(HASPPID) */
-
-		case 's':
-			Fsize = 1;
-			break;
-		case 'S':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    if (GOv) {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    }
-			    TmLimit = TMLIMIT;
-			    break;
-			}
-			for (cp = GOv, i = n = 0; *cp; cp++) {
-			    if (!isdigit((unsigned char)*cp))
-				break;
-			    i = (i * 10) + ((int)*cp - '0');
-			    n++;
-			}
-			if (n)
-			    TmLimit = i;
-			else
-			    TmLimit = TMLIMIT;
-			if (*cp) {
-			    GOx1 = GObk[0];
-			    GOx2 = GObk[1] + n;
-			}
-			if (TmLimit < TMLIMMIN) {
-			    (void) fprintf(stderr,
-				"%s: WARNING: -S time (%d) changed to %d\n",
-				Pn, TmLimit, TMLIMMIN);
-			    TmLimit = TMLIMMIN;
-			}
-			break;
-		case 't':
-			Fterse = Fwarn = 1;
-			break;
-		case 'T':
-			if (!GOv || *GOv == '-' || *GOv == '+') {
-			    if (GOv) {
-				GOx1 = GObk[0];
-				GOx2 = GObk[1];
-			    }
-			    Ftcptpi = (GOp == '-') ? 0 : TCPTPI_STATE;
-			    break;
-			}
-			for (Ftcptpi = 0; *GOv; GOv++) {
-			    switch (*GOv) {
-
-#if	defined(HASTCPTPIQ)
-			    case 'q':
-				Ftcptpi |= TCPTPI_QUEUES;
-				break;
-#endif	/* defined(HASTCPTPIQ) */
-
-			    case 's':
-				Ftcptpi |= TCPTPI_STATE;
-				break;
-
-#if	defined(HASTCPTPIW)
-			    case 'w':
-				Ftcptpi |= TCPTPI_WINDOWS;
-				break;
-#endif	/* defined(HASTCPTPIW) */
-
-			    default:
-				(void) fprintf(stderr,
-				"%s: unsupported TCP/TPI info selection: %c\n",
-				    Pn, *GOv);
-				err = 1;
-			    }
-			}
-			break;
-		case 'u':
-			if (enter_uid(GOv))
-			    err = 1;
-			break;
-		case 'U':
-			Funix = 1;
-			break;
-		case 'v':
-			version = 1;
-			break;
-		case 'V':
-			Fverbose = 1;
-			break;
-		case 'w':
-			Fwarn = (GOp == '+') ? 0 : 1;
-			break;
-
-#if	defined(HASXOPT)
-		case 'X':
-			Fxopt = Fxopt ? 0 : 1;
-			break;
-#endif	/* defined(HASXOPT) */
-
-		default:
-			(void) fprintf(stderr, "%s: unknown option (%c)\n",
-			    Pn, c);
+		    }
+		    if ( ! FieldSel[i].nm) {
+			(void) fprintf(stderr,
+			    "%s: unknown field: %c\n", Pn, *GOv);
+			err++;
+		    }
+		}
+		Ffield = 1;
+		break;
+	    case 'g':
+		if (GOv) {
+		    if (*GOv == '-' || *GOv == '+') {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    } else if (enter_id(PGRP, GOv))
 			err = 1;
 		}
+		Fpgrp = 1;
+		break;
+	    case 'h':
+	    case '?':
+		Fhelp = 1;
+		break;
+	    case 'i':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Fnet = 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    break;
+		}
+		if (enter_network_address(GOv))
+		    err = 1;
+		break;
+
+#if	defined(HASKOPT)
+	    case 'k':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    (void) fprintf(stderr, "%s: -k not followed by path\n", Pn);
+		    err = 1;
+		} else
+		    Nmlst = GOv;
+		break;
+#endif	/* defined(HASKOPT) */
+
+	    case 'l':
+		Futol = 0;
+		break;
+
+#if	defined(HASMOPT)
+	    case 'm':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    (void) fprintf(stderr, "%s: -m not followed by path\n", Pn);
+		    err = 1;
+		} else
+		    Memory = GOv;
+		break;
+#endif	/* defined(HASMOPT) */
+
+	    case 'M':
+		FportMap = (GOp == '+') ? 1 : 0;
+		break;
+	    case 'n':
+		Fhost = (GOp == '-') ? 0 : 1;
+		break;
+	    case 'N':
+		Fnfs = 1;
+		break;
+	    case 'o':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Foffset = 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    break;
+		}
+		for (cp = GOv, i = n = 0; *cp; cp++) {
+		    if (!isdigit((unsigned char)*cp))
+			break;
+		    i = (i * 10) + ((int)*cp - '0');
+		    n++;
+		}
+		if (n)
+		    OffDecDig = i;
+		else
+		    Foffset = 1;
+		if (*cp) {
+		    GOx1 = GObk[0];
+		    GOx2 = GObk[1] + n;
+		}
+		break;
+	    case 'O':
+		Fovhd = (GOp == '-') ? 1 : 0;
+		break;
+	    case 'p':
+		if (enter_id(PID, GOv))
+		    err = 1;
+		break;
+	    case 'P':
+		Fport = (GOp == '-') ? 0 : 1;
+		break;
+	    case 'r':
+		if (GOp == '+')
+		    ev = rc = 1;
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    RptTm = RPTTM;
+		    break;
+		}
+		for (cp = GOv, i = n = 0; *cp; cp++) {
+		    if (!isdigit((unsigned char)*cp))
+			break;
+		    i = (i * 10) + ((int)*cp - '0');
+		    n++;
+		}
+		if (n)
+		    RptTm = i;
+		else
+		    RptTm = RPTTM;
+		if (*cp) {
+		    GOx1 = GObk[0];
+		    GOx2 = GObk[1] + n;
+		}
+		break;
+
+#if	defined(HASPPID)
+	    case 'R':
+		Fppid = 1;
+		break;
+#endif	/* defined(HASPPID) */
+
+	    case 's':
+		Fsize = 1;
+		break;
+	    case 'S':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    TmLimit = TMLIMIT;
+		    break;
+		}
+		for (cp = GOv, i = n = 0; *cp; cp++) {
+		    if (!isdigit((unsigned char)*cp))
+			break;
+		    i = (i * 10) + ((int)*cp - '0');
+		    n++;
+		}
+		if (n)
+		    TmLimit = i;
+		else
+		    TmLimit = TMLIMIT;
+		if (*cp) {
+		    GOx1 = GObk[0];
+		    GOx2 = GObk[1] + n;
+		}
+		if (TmLimit < TMLIMMIN) {
+		    (void) fprintf(stderr,
+			"%s: WARNING: -S time (%d) changed to %d\n",
+			Pn, TmLimit, TMLIMMIN);
+		    TmLimit = TMLIMMIN;
+		}
+		break;
+	    case 't':
+		Fterse = Fwarn = 1;
+		break;
+	    case 'T':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    Ftcptpi = (GOp == '-') ? 0 : TCPTPI_STATE;
+		    break;
+		}
+		for (Ftcptpi = 0; *GOv; GOv++) {
+		    switch (*GOv) {
+
+#if	defined(HASTCPTPIQ)
+		    case 'q':
+			Ftcptpi |= TCPTPI_QUEUES;
+			break;
+#endif	/* defined(HASTCPTPIQ) */
+
+		    case 's':
+			Ftcptpi |= TCPTPI_STATE;
+			break;
+
+#if	defined(HASTCPTPIW)
+		    case 'w':
+			Ftcptpi |= TCPTPI_WINDOWS;
+			break;
+#endif	/* defined(HASTCPTPIW) */
+
+		    default:
+			(void) fprintf(stderr,
+			"%s: unsupported TCP/TPI info selection: %c\n",
+			    Pn, *GOv);
+			err = 1;
+		    }
+		}
+		break;
+	    case 'u':
+		if (enter_uid(GOv))
+		    err = 1;
+		break;
+	    case 'U':
+		Funix = 1;
+		break;
+	    case 'v':
+		version = 1;
+		break;
+	    case 'V':
+		Fverbose = 1;
+		break;
+	    case 'w':
+		Fwarn = (GOp == '+') ? 0 : 1;
+		break;
+
+#if	defined(HASXOPT)
+	    case 'X':
+		Fxopt = Fxopt ? 0 : 1;
+		break;
+#endif	/* defined(HASXOPT) */
+
+	    default:
+		(void) fprintf(stderr, "%s: unknown option (%c)\n", Pn, c);
+		err = 1;
+	    }
 	}
 	if (Fsize && Foffset) {
-		(void) fprintf(stderr, "%s: -o and -s are mutually exclusive\n",
-			Pn);
-		err++;
+	    (void) fprintf(stderr, "%s: -o and -s are mutually exclusive\n",
+		Pn);
+	    err++;
 	}
 	if (Ffield) {
-		if (Fterse) {
-			(void) fprintf(stderr,
-				"%s: -f and -t are mutually exclusive\n", Pn);
-			err++;
-		}
-		FieldSel[LSOF_FIX_PID].st = 1;
+	    if (Fterse) {
+		(void) fprintf(stderr,
+		    "%s: -f and -t are mutually exclusive\n", Pn);
+		err++;
+	    }
+	    FieldSel[LSOF_FIX_PID].st = 1;
 	}
 	if (DChelp || err || Fhelp || fh || version)
-		usage(err ? 1 : 0, fh, version);
+	    usage(err ? 1 : 0, fh, version);
 /*
  * Reduce the size of Suid[], if necessary.
  */
 	if (Suid && Nuid && Nuid < Mxuid) {
-		if (!(Suid = (struct seluid *)realloc((MALLOC_P *)Suid,
-				(MALLOC_S)(sizeof(struct seluid) * Nuid))))
-		{
-			(void) fprintf(stderr, "%s: can't realloc UID table\n",
-				Pn);
-			Exit(1);
-		}
-		Mxuid = Nuid;
+	    if (!(Suid = (struct seluid *)realloc((MALLOC_P *)Suid,
+			 (MALLOC_S)(sizeof(struct seluid) * Nuid))))
+	    {
+		(void) fprintf(stderr, "%s: can't realloc UID table\n", Pn);
+		Exit(1);
+	    }
+	    Mxuid = Nuid;
 	}
-
 /*
  * Compute the selection flags.
  */
 	if (Cmdl)
-		Selflags |= SELCMD;
+	    Selflags |= SELCMD;
 	if (Fdl)
 		Selflags |= SELFD;
 	if (Fnet)
-		Selflags |= SELNET;
+	    Selflags |= SELNET;
 	if (Fnfs)
-		Selflags |= SELNFS;
+	    Selflags |= SELNFS;
 	if (Funix)
-		Selflags |= SELUNX;
+	    Selflags |= SELUNX;
 	if (Npgrp)
-		Selflags |= SELPGRP;
+	    Selflags |= SELPGRP;
 	if (Npid)
-		Selflags |= SELPID;
+	    Selflags |= SELPID;
 	if (Nuid && Nuidincl)
-		Selflags |= SELUID;
+	    Selflags |= SELUID;
 	if (Nwad)
-		Selflags |= SELNA;
+	    Selflags |= SELNA;
 	if (GOx1 < argc)
-		Selflags |= SELNM;
+	    Selflags |= SELNM;
 	if (Selflags == 0) {
-		if (Fand) {
-			(void) fprintf(stderr,
-				"%s: no select options to AND via -a\n", Pn);
-			usage(1, 0, 0);
-		}
-		Selflags = SELALL;
+	    if (Fand) {
+		(void) fprintf(stderr,
+		    "%s: no select options to AND via -a\n", Pn);
+		usage(1, 0, 0);
+	    }
+	    Selflags = SELALL;
 	} else {
-		if (GOx1 >= argc && (Selflags & (SELNA|SELNET)) != 0
-		&&  (Selflags & ~(SELNA|SELNET)) == 0)
-			Selinet = 1;
-		Selall = 0;
+	    if (GOx1 >= argc && (Selflags & (SELNA|SELNET)) != 0
+	    &&  (Selflags & ~(SELNA|SELNET)) == 0)
+		Selinet = 1;
+	    Selall = 0;
 	}
 /*
  * Read the mount table, unless listing only network files (-i option).
  */
 	if (Selinet == 0 && readmnt() == 0)
-		Exit(1);
+	    Exit(1);
 /*
  * Process the file arguments.
  */
 	if (GOx1 < argc) {
-		if (ck_file_arg(GOx1, argc, argv))
-			usage(1, 0, 0);
+	    if (ck_file_arg(GOx1, argc, argv))
+		usage(1, 0, 0);
 	}
 /*
  * Do dialect-specific initialization.
@@ -573,7 +570,7 @@ main(argc, argv)
  * If there is a device cache, prepare the device table.
  */
 	if (DCstate)
-		readdev(0);
+	    readdev(0);
 #endif	/* defined(HASDCACHE) */
 
 /*
@@ -591,7 +588,7 @@ main(argc, argv)
  * Gather and report process information every RptTm seconds.
  */
 	if (RptTm)
-		CkPasswd = 1;
+	    CkPasswd = 1;
 	do {
 
 	/*
@@ -599,24 +596,21 @@ main(argc, argv)
 	 */
 	    gather_proc_info();
 	/*
-	 * Select the information to be displayed and sort it by Process ID,
-	 * unless sorting is inhibited.  In that case, the gather_proc_info()
-	 * function will have produced output as it became available.
+	 * If the local process table has more than one entry, sort it by PID.
 	 */
 	    if (Nlproc > 1) {
-		if (!slp) {
-		    slp = (struct lproc **)malloc(
-			  (MALLOC_S)(sizeof(struct lproc *) * Nlproc));
+		if (Nlproc > sp) {
+		    len = (MALLOC_S)(Nlproc * sizeof(struct lproc *));
 		    sp = Nlproc;
-		} else if (Nlproc > sp) {
-		    slp = (struct lproc **)realloc(slp,
-			  (MALLOC_S)(sizeof(struct lproc *) * Nlproc));
-		    sp = Nlproc;
-		}
-		if (!slp) {
-		    (void) fprintf(stderr,
-			"%s: no space for %d sort pointers\n", Pn, Nlproc);
-		    Exit(1);
+		    if (!slp)
+			slp = (struct lproc **)malloc(len);
+		    else
+			slp = (struct lproc **)realloc((MALLOC_P *)slp, len);
+		    if (!slp) {
+			(void) fprintf(stderr,
+			    "%s: no space for %d sort pointers\n", Pn, Nlproc);
+			Exit(1);
+		    }
 		}
 		for (i = 0; i < Nlproc; i++) {
 		    slp[i] = &Lproc[i];
@@ -624,18 +618,22 @@ main(argc, argv)
 		(void) qsort((QSORT_P *)slp, (size_t)Nlproc,
 			     (size_t)sizeof(struct lproc *), comppid);
 	    }
-	    if (Nlproc) {
+	    if ((n = Nlproc)) {
 
 	    /*
+	     * Print the selected processes and count them.
+	     *
 	     * Lf contents must be preserved, since they may point to a
 	     * malloc()'d area, and since Lf is used throughout the print
 	     * process.
 	     */
 		for (lf = Lf, print_init(); PrPass < 2; PrPass++) {
-		    for (i = 0; i < Nlproc; i++) {
+		    for (i = n = 0; i < Nlproc; i++) {
 			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
-			if (Lp->pss)
+			if (Lp->pss) {
 			    print_proc();
+			    n++;
+			}
 			if (RptTm && PrPass)
 			    (void) free_lproc(Lp);
 		    }
@@ -644,8 +642,16 @@ main(argc, argv)
 	    }
 	/*
 	 * If a repeat time is set, sleep for the specified time.
+	 *
+	 * If conditional repeat mode is in effect, see if it's time to exit.
 	 */
 	    if (RptTm) {
+		if (rc) {
+		    if (!n)
+			break;
+		    else
+			ev = 0;
+		}
 		if (Ffield) {
 		    putchar(LSOF_FID_MARK);
 		    putchar('\n');
@@ -672,8 +678,10 @@ main(argc, argv)
 	    if (str->f)
 		continue;
 	    rv = 1;
-	    if (Fverbose)
-		(void) printf("%s: command not located: %s\n", Pn, str->str);
+	    if (Fverbose) {
+		(void) printf("%s: command not located: ", Pn);
+		safestrprt(str->str, stdout, 1);
+	    }
 	}
 	for (sfp = Sfile; sfp; sfp = sfp->next) {
 
@@ -683,11 +691,11 @@ main(argc, argv)
 	    if (sfp->f)
 		continue;
 	    rv = 1;
-	    if (Fverbose)
-		(void) printf("%s: no file%s use located: %s\n",
-		    Pn,
-		    sfp->type ? "" : " system",
-		    sfp->aname);
+	    if (Fverbose) {
+		(void) printf("%s: no file%s use located: ", Pn,
+		    sfp->type ? "" : " system");
+		safestrprt(sfp->aname, stdout, 1);
+	    }
 	}
 
 #if	defined(HASPROCFS)
@@ -696,9 +704,10 @@ main(argc, argv)
 	 */
 	    if (Procsrch && !Procfind) {
 		rv = 1;
-		if (Fverbose)
-		    (void) printf("%s: no file system use located: %s\n",
-		    Pn, Mtprocfs ? Mtprocfs->dir : HASPROCFS);
+		if (Fverbose) {
+		    (void) printf("%s: no file system use located: ", Pn);
+		    safestrprt(Mtprocfs ? Mtprocfs->dir : HASPROCFS, stdout, 1);
+		}
 	    }
 	    {
 		struct procfsid *pfi;
@@ -706,9 +715,10 @@ main(argc, argv)
 		for (pfi = Procfsid; pfi; pfi = pfi->next) {
 		    if (!pfi->f) {
 			rv = 1;
-			if (Fverbose)
-			    (void) printf("%s: no file use located: %s\n",
-				Pn, pfi->nm);
+			if (Fverbose) {
+			    (void) printf("%s: no file use located: ", Pn);
+			    safestrprt(pfi->nm, stdout, 1);
+			}
 		    }
 		}
 	    }
@@ -729,9 +739,10 @@ main(argc, argv)
 		    break;
 	    }
 	    rv |= i;
-	    if (i && Fverbose)
-		(void) printf("%s: Internet address not located: %s\n",
-		    Pn, cp ? cp : "(unknown)");
+	    if (i && Fverbose) {
+		(void) printf("%s: Internet address not located: ", Pn);
+		safestrprt(cp ? cp : "(unknown)", stdout, 1);
+	    }
 	}
 	if (Fnet && Fnet < 2) {
 
@@ -784,14 +795,17 @@ main(argc, argv)
 		continue;
 	    rv = 1;
 	    if (Fverbose) {
-		if (Suid[i].lnm)
-		    (void) printf("%s: login name (UID %lu) not located: %s\n",
-			Pn, (unsigned long)Suid[i].uid, Suid[i].lnm);
-		else
+		if (Suid[i].lnm) {
+		    (void) printf("%s: login name (UID %lu) not located: ",
+			Pn, (unsigned long)Suid[i].uid);
+		    safestrprt(Suid[i].lnm, stdout, 1);
+		} else
 		    (void) printf("%s: user ID not located: %lu\n", Pn,
 			(unsigned long)Suid[i].uid);
 	    }
 	}
+	if (!rv && rc)
+	    rv = ev;
 	Exit(rv);
 }
 
