@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.9 98/01/29 15:38:48 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.10 98/03/06 07:25:38 abe Exp $";
 #endif
 
 
@@ -66,6 +66,9 @@ _PROTOTYPE(static int handleint,(int sig));
 _PROTOTYPE(static void handleint,(int sig));
 #endif	/* defined(HASINTSIGNAL) */
 
+_PROTOTYPE(static void safecpy,(char *d, char *s, int sc));
+_PROTOTYPE(static void safecpyn,(char *d, char *s, MALLOC_S sl, int sc));
+
 
 /*
  * Local variables
@@ -164,96 +167,6 @@ closePipes()
 }
 
 
-
-/*
- * Exit() - do a clean exit()
- */
-
-void
-Exit(xv)
-	int xv;				/* exit() value */
-{
-	(void) childx();
-
-#if	defined(HASDCACHE)
-	if (DCrebuilt && !Fwarn)
-	    (void) fprintf(stderr, "%s: WARNING: %s was updated.\n",
-		Pn, DCpath[DCpathX]);
-#endif	/* defined(HASDCACHE) */
-
-	exit(xv);
-}
-
-
-#if	defined(WILLDROPGID)
-/*
- * dropgid() - drop setgid permission
- */
-
-void
-dropgid()
-{
-	if (!Setuidroot && Setgid) {
-		if (setgid(Mygid) < 0) {
-			(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
-				Pn, (int)Mygid, strerror(errno));
-			Exit(1);
-		}
-		Setgid = 0;
-	}
-}
-#endif	/* defined(WILLDROPGID) */
-
-
-#if	defined(HASNLIST)
-/*
- * get_Nl_value() - get Nl value for nickname
- */
-
-int
-get_Nl_value(nn, d, v)
-	char *nn;			/* nickname of requested entry */
-	struct drive_Nl *d;		/* drive_Nl table that built Nl
-					 * (if NULL, use Build_Nl) */
-	unsigned long *v;		/* returned value (if NULL,
-					 * return nothing) */
-{
-	int i;
-
-	if (!Nl || !Nll)
-	    return(-1);
-	if (!d)
-	    d = Build_Nl;
-	for (i = 0; d->nn; d++, i++) {
-	    if (strcmp(d->nn, nn) == 0) {
-		if (v)
-		    *v = (unsigned long)Nl[i].n_value;
-		return(i);
-	    }
-	}
-	return(-1);
-}
-#endif	/* defined(HASNLIST) */
-
-
-/*
- * is_readable() -- is file readable
- */
-
-int
-is_readable(path, msg)
-	char *path;			/* file path */
-	int msg;			/* issue warning message if 1 */
-{
-	if (access(path, R_OK) < 0) {
-	    if (!Fwarn && msg == 1)
-		(void) fprintf(stderr, ACCESSERRFMT, Pn, path, strerror(errno));
-	    return(0);
-	}
-	return(1);
-}
-
-
 /*
  * compdev() - compare Devtp[] entries
  */
@@ -454,6 +367,20 @@ doinchild(fn, fp, rbuf, rbln)
 }
 
 
+/*
+ * doreadlink() -- do a readlink() function
+ */
+
+static int
+doreadlink(path, rbuf, rbln)
+	char *path;			/* path */
+	char *rbuf;			/* response buffer */
+	int rbln;			/* response buffer length */
+{
+	return(readlink(path, rbuf, rbln));
+}
+
+
 /*
  * dostat() - do a *stat() function
  *
@@ -479,18 +406,24 @@ dostat(path, rbuf, rbln)
 }
 
 
+#if	defined(WILLDROPGID)
 /*
- * doreadlink() -- do a readlink() function
+ * dropgid() - drop setgid permission
  */
 
-static int
-doreadlink(path, rbuf, rbln)
-	char *path;			/* path */
-	char *rbuf;			/* response buffer */
-	int rbln;			/* response buffer length */
+void
+dropgid()
 {
-	return(readlink(path, rbuf, rbln));
+	if (!Setuidroot && Setgid) {
+		if (setgid(Mygid) < 0) {
+			(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
+				Pn, (int)Mygid, strerror(errno));
+			Exit(1);
+		}
+		Setgid = 0;
+	}
 }
+#endif	/* defined(WILLDROPGID) */
 
 
 /*
@@ -501,20 +434,18 @@ void
 enter_dev_ch(m)
 	char *m;
 {
-	MALLOC_S len;
 	char *mp;
 
 	if (!m || *m == '\0')
-		return;
-	len = strlen(m) + 1;
-	if (!(mp = (char *)malloc(len))) {
-		(void) fprintf(stderr, "%s: no more dev_ch space at PID %d\n",
-			Pn, Lp->pid);
-		Exit(1);
+	    return;
+	if (!(mp = mkstrcpy(m, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no more dev_ch space at PID %d: \n",
+		Pn, Lp->pid);
+	    safestrprt(m, stderr, 1);
+	    Exit(1);
 	}
-	(void) strcpy(mp, m);
 	if (Lf->dev_ch)
-		(void) free((FREE_P *)Lf->dev_ch);
+	   (void) free((FREE_P *)Lf->dev_ch);
 	Lf->dev_ch = mp;
 }
 
@@ -527,24 +458,73 @@ void
 enter_nm(m)
 	char *m;
 {
-	MALLOC_S len;
 	char *mp;
 
 	if (!m || *m == '\0')
-		return;
-	len = strlen(m) + 1;
-	if (!(mp = (char *)malloc(len))) {
-		(void) fprintf(stderr, "%s: no more nm space at PID %d\n",
-			Pn, Lp->pid);
-		Exit(1);
+	    return;
+	if (!(mp = mkstrcpy(m, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no more nm space at PID %d for: ",
+		Pn, Lp->pid);
+	    safestrprt(m, stderr, 1);
+	    Exit(1);
 	}
-	(void) strcpy(mp, m);
 	if (Lf->nm)
-		(void) free((FREE_P *)Lf->nm);
+	    (void) free((FREE_P *)Lf->nm);
 	Lf->nm = mp;
 }
 
 
+/*
+ * Exit() - do a clean exit()
+ */
+
+void
+Exit(xv)
+	int xv;				/* exit() value */
+{
+	(void) childx();
+
+#if	defined(HASDCACHE)
+	if (DCrebuilt && !Fwarn)
+	    (void) fprintf(stderr, "%s: WARNING: %s was updated.\n",
+		Pn, DCpath[DCpathX]);
+#endif	/* defined(HASDCACHE) */
+
+	exit(xv);
+}
+
+
+#if	defined(HASNLIST)
+/*
+ * get_Nl_value() - get Nl value for nickname
+ */
+
+int
+get_Nl_value(nn, d, v)
+	char *nn;			/* nickname of requested entry */
+	struct drive_Nl *d;		/* drive_Nl table that built Nl
+					 * (if NULL, use Build_Nl) */
+	KA_T *v;			/* returned value (if NULL,
+					 * return nothing) */
+{
+	int i;
+
+	if (!Nl || !Nll)
+	    return(-1);
+	if (!d)
+	    d = Build_Nl;
+	for (i = 0; d->nn; d++, i++) {
+	    if (strcmp(d->nn, nn) == 0) {
+		if (v)
+		    *v = (KA_T)Nl[i].n_value;
+		return(i);
+	    }
+	}
+	return(-1);
+}
+#endif	/* defined(HASNLIST) */
+
+
 /*
  * handleint() - handle an interrupt
  */
@@ -600,6 +580,113 @@ is_nw_addr(ia, p)
 }
 
 
+/*
+ * mkstrcpy() - make a string copy in malloc()'d space
+ *
+ * return: copy pointer
+ *	   copy length (optional)
+ */
+
+char *
+mkstrcpy(src, rlp)
+	char *src;			/* source */
+	MALLOC_S *rlp;			/* returned length pointer (optional)
+					 * The reurend length is an strlen()
+					 * equivalent */
+{
+	MALLOC_S len;
+	char *ns;
+
+	len = (MALLOC_S)(src ? strlen(src) : 0);
+	ns = (char *)malloc(len + 1);
+	if (ns) {
+	    if (src)
+		(void) strcpy(ns, src);
+	    else
+		*ns = '\0';
+	}
+	if (rlp)
+	    *rlp = len;
+	return(ns);
+}
+
+
+/*
+ * mkstrcat() - make a catenated copy of up to three strings under optional
+ *		string-by-string count control
+ *
+ * return: copy pointer
+ *	   copy string length (optional)
+ */
+
+char *
+mkstrcat(s1, l1, s2, l2, s3, l3, clp)
+	char *s1;			/* source string 1 */
+	int l1;				/* length of string 1 (-1 if none) */
+	char *s2;			/* source string 2 */
+	int l2;				/* length of string 2 (-1 if none) */
+	char *s3;			/* source string 3 (optional) */
+	int l3	;			/* length of string 3 (-1 if none) */
+	MALLOC_S *clp;			/* pointer to return of copy length
+					 * (optional) */
+{
+	MALLOC_S cl, len1, len2, len3;
+	char *cp;
+
+	if (s1)
+	    len1 = (MALLOC_S)((l1 >= 0) ? l1 : strlen(s1));
+	else
+	    len1 = (MALLOC_S)0;
+	if (s2)
+	    len2 = (MALLOC_S)((l2 >= 0) ? l2 : strlen(s2));
+	else
+	    len2 = (MALLOC_S)0;
+	if (s3)
+	    len3 = (MALLOC_S)((l3 >= 0) ? l3 : strlen(s3));
+	else
+	    len3 = (MALLOC_S)0;
+	cl = len1 + len2 + len3;
+	if ((cp = (char *)malloc(cl + 1))) {
+	    char *tp = cp;
+
+	    if (s1 && len1) {
+		(void) strncpy(tp, s1, len1);
+		tp += len1;
+	    }
+	    if (s2 && len2) {
+		(void) strncpy(tp, s2, len2);
+		tp += len2;
+	    }
+	    if (s3 && len3) {
+		(void) strncpy(tp, s3, len3);
+		tp += len3;
+	    }
+	    *tp = '\0';
+	}
+	if (clp)
+	    *clp = cl;
+	return(cp);
+}
+
+
+/*
+ * is_readable() -- is file readable
+ */
+
+int
+is_readable(path, msg)
+	char *path;			/* file path */
+	int msg;			/* issue warning message if 1 */
+{
+	if (access(path, R_OK) < 0) {
+	    if (!Fwarn && msg == 1)
+		(void) fprintf(stderr, ACCESSERRFMT, Pn, path, strerror(errno));
+	    return(0);
+	}
+	return(1);
+}
+
+
 /*
  * Readlink() - read and interpret file system symbolic links
  */
@@ -623,51 +710,53 @@ Readlink(arg)
  * See if avoiding kernel blocks.
  */
 	if (Fblock) {
-		if (!Fwarn)
-		    (void) fprintf(stderr,
-			"%s: avoiding readlink(%s): -b was specified.\n",
-			Pn, arg);
-		return(arg);
+	    if (!Fwarn) {
+		(void) fprintf(stderr, "%s: avoiding readlink(", Pn);
+		safestrprt(arg, stderr, 0);
+		(void) fprintf(stderr, "): -b was specified.\n");
+	    }
+	    return(arg);
 	}
 /*
  * Evaluate each component of the argument for a symbolic link.
  */
 	for (alen = 0, ap = abuf, argp1 = argp2 = arg; *argp2; argp1 = argp2 ) {
-		for (argp2 = argp1 + 1; *argp2 && *argp2 != '/'; argp2++)
-			;
-		if ((len = argp2 - arg) >= MAXPATHLEN) {
+	    for (argp2 = argp1 + 1; *argp2 && *argp2 != '/'; argp2++)
+		;
+	    if ((len = argp2 - arg) >= MAXPATHLEN) {
 
 path_too_long:
-			if (!Fwarn) {
-				(void) fprintf(stderr,
-					"%s: path too long: %s\n", Pn, arg);
-			}
-			return((char *)NULL);
+		if (!Fwarn) {
+		    (void) fprintf(stderr,
+			"%s: readlink() path too long: ", Pn);
+		    safestrprt(arg, stderr, 1);
 		}
-		(void) strncpy(tbuf, arg, len);
-		tbuf[len] = '\0';
+		return((char *)NULL);
+	    }
+	    (void) strncpy(tbuf, arg, len);
+	    tbuf[len] = '\0';
 	/*
 	 * Dereference a symbolic link.
 	 */
-		if ((llen = doinchild(doreadlink, tbuf, lbuf, sizeof(lbuf) - 1))
-		>= 0) {
-		/*
-		 * If the link is a new absolute path, replace
-		 * the previous assembly with it.
-		 */
-			if (lbuf[0] == '/') {
-				(void) strncpy(abuf, lbuf, llen);
-				ap = &abuf[llen];
-				*ap = '\0';
-				alen = llen;
-				continue;
-			}
-			lbuf[llen] = '\0';
-			s1 = lbuf;
-		} else {
-			llen = argp2 - argp1;
-			s1 = argp1;
+	    if ((llen=doinchild(doreadlink,tbuf,lbuf,sizeof(lbuf) - 1)) >= 0) {
+
+	    /*
+	     * If the link is a new absolute path, replace
+	     * the previous assembly with it.
+	     */
+		if (lbuf[0] == '/') {
+		    (void) strncpy(abuf, lbuf, llen);
+		    ap = &abuf[llen];
+		    *ap = '\0';
+		    alen = llen;
+		    continue;
 		}
+		lbuf[llen] = '\0';
+		s1 = lbuf;
+	    } else {
+		llen = argp2 - argp1;
+		s1 = argp1;
+	    }
 	/*
 	 * Make sure two components are separated by a `/'.
 	 *
@@ -677,75 +766,74 @@ path_too_long:
 	 * If the first component is a link and the source of
 	 * the link has a leading '/', force a leading '/'.
 	 */
-		if (*s1 == '/') {
+	    if (*s1 == '/')
+		slen = 1;
+	    else {
+		if (alen > 0) {
+
+		/*
+		 * This is not the first component.
+		 */
+		    if (abuf[alen - 1] == '/')
 			slen = 1;
+		    else
+			slen = 2;
 		} else {
-			if (alen > 0) {
 
-			/*
-			 * This is not the first component.
-			 */
-				if (abuf[alen - 1] == '/')
-					slen = 1;
-				else
-					slen = 2;
-			} else {
-
-			/*
-			 * This is the first component.
-			 */
-				if (s1 == lbuf && tbuf[0] == '/')
-					slen = 2;
-				else
-					slen = 1;
-			}
+		/*
+		 * This is the first component.
+		 */
+		    if (s1 == lbuf && tbuf[0] == '/')
+			slen = 2;
+		    else
+			slen = 1;
 		}
+	    }
 	/*
 	 * Add to the path assembly.
 	 */
-		if ((alen + llen + slen) >= (int)sizeof(abuf))
-			goto path_too_long;
-		if (slen == 2)
-			*ap++ = '/';
-		(void) strncpy(ap, s1, llen);
-		ap += llen;
-		*ap = '\0';
-		alen += (llen + slen - 1);
+	    if ((alen + llen + slen) >= (int)sizeof(abuf))
+		goto path_too_long;
+	    if (slen == 2)
+		*ap++ = '/';
+	    (void) strncpy(ap, s1, llen);
+	    ap += llen;
+	    *ap = '\0';
+	    alen += (llen + slen - 1);
 	}
 /*
  * If the assembled path and argument are the same, free all but the
  * last string in the stack, and return the argument.
  */
 	if (strcmp(arg, abuf) == 0) {
-		for (i = 0; i < sx; i++) {
-			if (i < (sx - 1))
-				(void) free((FREE_P *)stk[i]);
-			stk[i] = (char *)NULL;
-		}
-		sx = 0;
-		return(arg);
+	    for (i = 0; i < sx; i++) {
+		if (i < (sx - 1))
+		    (void) free((FREE_P *)stk[i]);
+		stk[i] = (char *)NULL;
+	    }
+	    sx = 0;
+	    return(arg);
 	}
 /*
  * If the assembled path and argument are different, add it to the
  * string stack, then Readlink() it.
  */
-	if (!(s1 = (char *)malloc((MALLOC_S)(alen + 1)))) {
+	if (!(s1 = mkstrcpy(abuf, (MALLOC_S *)NULL))) {
 
 no_readlink_space:
 
-		(void) fprintf(stderr,
-			"%s: no Readlink string space for %s\n", Pn, abuf);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: no Readlink string space for ", Pn);
+	    safestrprt(abuf, stderr, 1);
+	    Exit(1);
 	}
-	(void) strcpy(s1, abuf);
 	if (++sx > ss) {
-		if (!stk)
-			stk = (char **)malloc(sizeof(char *) * sx);
-		else
-			stk = (char **)realloc(stk, sizeof(char *) * sx);
-		if (!stk)
-			goto no_readlink_space;
-		ss = sx;
+	    if (!stk)
+		stk = (char **)malloc(sizeof(char *) * sx);
+	    else
+		stk = (char **)realloc(stk, sizeof(char *) * sx);
+	    if (!stk)
+		goto no_readlink_space;
+	    ss = sx;
 	}
 	stk[sx - 1] = s1;
 	return(Readlink(s1));
@@ -759,14 +847,14 @@ no_readlink_space:
 
 int
 readstdata(addr, buf)
-	struct stdata *addr;		/* stdata address in kernel*/
+	KA_T addr;			/* stdata address in kernel*/
 	struct stdata *buf;		/* buffer addess */
 {
 	if (!addr
-	||  kread((KA_T)addr, (char *)buf, sizeof(struct stdata))) {
-		(void) sprintf(Namech, "no stream data in %#lx\n",
-			(unsigned long)addr);
-		return(1);
+	||  kread(addr, (char *)buf, sizeof(struct stdata))) {
+	    (void) sprintf(Namech, "no stream data in %s",
+		print_kptr(addr, (char *)NULL));
+	    return(1);
 	}
 	return(0);
 }
@@ -778,21 +866,21 @@ readstdata(addr, buf)
 
 int
 readsthead(addr, buf)
-	struct queue *addr;		/* starting queue pointer in kernel */
+	KA_T addr;			/* starting queue pointer in kernel */
 	struct queue *buf;		/* buffer for queue head */
 {
-	struct queue *qp;
+	KA_T qp;
 
 	if (!addr) {
-		(void) strcpy(Namech, "no stream queue head");
-		return(1);
+	    (void) strcpy(Namech, "no stream queue head");
+	    return(1);
 	}
-	for (qp = addr; qp; qp = buf->q_next) {
-		if (kread((KA_T)qp, (char *)buf, sizeof(struct queue))) {
-			(void) sprintf(Namech, "bad stream queue link at %#lx",
-				(unsigned long)qp);
-			return(1);
-		}
+	for (qp = addr; qp; qp = (KA_T)buf->q_next) {
+	    if (kread(qp, (char *)buf, sizeof(struct queue))) {
+		(void) sprintf(Namech, "bad stream queue link at %s",
+		    print_kptr(qp, (char *)NULL));
+		return(1);
+	    }
 	}
 	return(0);
 }
@@ -804,14 +892,14 @@ readsthead(addr, buf)
 
 int
 readstidnm(addr, buf, len)
-	char *addr;			/* module ID name address in kernel */
+	KA_T addr;			/* module ID name address in kernel */
 	char *buf;			/* receiving buffer address */
 	READLEN_T len;			/* buffer length */
 {
-	if (!addr ||  kread((KA_T)addr, buf, len)) {
-		(void) sprintf(Namech, "can't read module ID name from %#lx",
-			(unsigned long)addr);
-		return(1);
+	if (!addr || kread(addr, buf, len)) {
+	    (void) sprintf(Namech, "can't read module ID name from %s",
+		print_kptr(addr, (char *)NULL));
+	    return(1);
 	}
 	return(0);
 }
@@ -823,14 +911,13 @@ readstidnm(addr, buf, len)
 
 int
 readstmin(addr, buf)
-	struct module_info *addr;	/* module info address in kernel */
+	KA_T addr;			/* module info address in kernel */
 	struct module_info *buf;	/* receiving buffer address */
 {
-	if (!addr
-	||  kread((KA_T)addr, (char *)buf, sizeof(struct module_info))) {
-		(void) sprintf(Namech, "can't read module info from %#lx",
-			(unsigned long)addr);
-		return(1);
+	if (!addr || kread(addr, (char *)buf, sizeof(struct module_info))) {
+	    (void) sprintf(Namech, "can't read module info from %s",
+		print_kptr(addr, (char *)NULL));
+	    return(1);
 	}
 	return(0);
 }
@@ -842,20 +929,105 @@ readstmin(addr, buf)
 
 int
 readstqinit(addr, buf)
-	struct qinit *addr;		/* queue info address in kernel */
+	KA_T addr;			/* queue info address in kernel */
 	struct qinit *buf;		/* receiving buffer address */
 {
-	if (!addr
-	||  kread((KA_T)addr, (char *)buf, sizeof(struct qinit))) {
-		(void) sprintf(Namech, "can't read queue info from %#lx",
-			(unsigned long)addr);
-		return(1);
+	if (!addr || kread(addr, (char *)buf, sizeof(struct qinit))) {
+	    (void) sprintf(Namech, "can't read queue info from %s",
+		print_kptr(addr, (char *)NULL));
+	    return(1);
 	}
 	return(0);
 }
 #endif	/* HASSTREAMS */
 
 
+/*
+ * safestrlen() - calculate a "safe" string length -- i.e., compute space
+ *		  for non-printable characters when printf'd as "\x%02x"
+ */
+
+int
+safestrlen(sp)
+	char *sp;			/* string pointer */
+{
+	int len = 0;
+
+	if (sp) {
+	    while (*sp) {
+		if (!isprint((unsigned char)*sp++))
+		    len += 4;		/* length of "\x%02x" printf */
+		else
+		    len++;
+	    }
+	}
+	return(len);
+}
+
+
+/*
+ * safestrprt() - print a string "safely" to the indicated stream -- i.e.,
+ *		  print unprintable characters in "\x%02x" printf form
+ */
+
+void
+safestrprt(sp, fs, nl)
+	char *sp;			/* string to print pointer pointer */
+	FILE *fs;			/* destination stream -- e.g., stderr
+					 * or stdout */
+	int nl;				/* 1 if trailing NL required */
+{
+	if (sp) {
+	    for (; *sp; sp++) {
+		if (isprint((unsigned char)*sp))
+		    putc((int)(*sp & 0xff), fs);
+		else
+		    (void) fprintf(fs, "\\x%02x", (unsigned int)(*sp & 0xff));
+	    }
+	}
+	if (nl)
+	    putc('\n', fs);
+}
+
+
+/*
+ * safestrprtn() - print a specified number of characters from a string
+ *		   "safely" to the indicated stream -- i.e.,  print
+ *		   unprintable characters in "\x%02x" printf form
+ */
+
+void
+safestrprtn(sp, len, fs, nl)
+	char *sp;			/* string to print pointer pointer */
+	int len;			/* safe number of characters to
+					 * print */
+	FILE *fs;			/* destination stream -- e.g., stderr
+					 * or stdout */
+	int nl;				/* 1 if trailing NL required */
+{
+	int i;
+
+	if (sp) {
+	    for (i = 0; i < len && *sp; sp++) {
+		if (isprint((unsigned char)*sp)) {
+		    putc((int)(*sp & 0xff), fs);
+		    i++;
+		} else {
+		    if ((i + 4) > len)
+			break;
+		    (void) fprintf(fs, "\\x%02x", (unsigned int)(*sp & 0xff));
+		    i += 4;
+		}
+	    }
+	} else
+	    i = 0;
+	for (; i < len; i++)
+	    putc(' ', fs);
+	if (nl)
+	    putc('\n', fs);
+}
+
+
 /*
  * statsafely() - stat path safely (i. e., with timeout)
  */
