@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.14 98/01/29 15:40:13 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.15 98/03/06 07:27:47 abe Exp $";
 #endif
 
 
@@ -217,26 +217,29 @@ fill_portmap()
 		(void) sprintf(buf, "%lu", p->pml_map.pm_prog);
 		cp = buf;
 	    }
-	    if (!(nl = strlen(cp)))
+	    if (!strlen(cp))
 		continue;
 	/*
 	 * Allocate space for the portmap name entry and copy it there.
 	 */
-	    if (!(nm = (char *)malloc(nl + 1))) {
+	    if (!(nm = mkstrcpy(cp, &nl))) {
 		(void) fprintf(stderr,
-		    "%s: can't allocate %d bytes for portmap entry: %s\n",
-		    Pn, (int)(nl + 1), cp);
+		    "%s: can't allocate space for portmap entry: ", Pn);
+		safestrprt(cp, stderr, 1);
 		Exit(1);
 	    }
-	    (void) strcpy(nm, cp);
+	    if (!nl) {
+		(void) free((FREE_P *)nm);
+		continue;
+	    }
 	/*
 	 * Allocate and fill a porttab struct entry for the portmap table.
 	 * Link it to the head of its hash bucket, and make it the new head.
 	 */
 	    if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
 		(void) fprintf(stderr,
-		    "%s: can't allocate porttab entry for protmap: %s\n",
-		    Pn, nm);
+		    "%s: can't allocate porttab entry for portmap: ", Pn);
+		safestrprt(nm, stderr, 1);
 		Exit(1);
 	    }
 	    pt->name = nm;
@@ -279,7 +282,7 @@ fill_porttab()
 		pr = 1;
 	    else
 		continue;
-	    if (!(nl = (MALLOC_S)strlen(se->s_name)))
+	    if (!se->s_name || !strlen(se->s_name))
 		continue;
 	    p = ntohs(se->s_port);
 	/*
@@ -296,21 +299,24 @@ fill_porttab()
 	/*
 	 * Add a new entry to the cache for this port and protocol.
 	 */
+	    if (!(nm = mkstrcpy(se->s_name, &nl))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for port %d name: %s\n",
+		    Pn, (int)(nl + 1), p, se->s_name);
+		Exit(1);
+	    }
+	    if (!nl) {
+		(void) free((FREE_P *)nm);
+		continue;
+	    }
 	    if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate porttab entry for port %d: %s\n",
 		    Pn, p, se->s_name);
 		Exit(1);
 	    }
-	    if (!(nm = (char *)malloc(nl+1))) {
-		(void) fprintf(stderr,
-		    "%s: can't allocate %d bytes for port %d name: %s\n",
-		    Pn, (int)(nl + 1), p, se->s_name);
-		Exit(1);
-	    }
-	    (void) strcpy(nm, se->s_name);
 	    pt->name = nm;
-	    pt->nl = nl;
+	    pt->nl = nl - 1;
 	    pt->port = p;
 	    pt->next = Pth[pr][h];
 	    pt->ss = 0;
@@ -334,6 +340,7 @@ gethostnm(ia)
 	char *hn, *np;
 	int i;
 	struct hostent *he = (struct hostent *)NULL;
+	MALLOC_S len;
 	static int nhc = 0;
 	unsigned char *u;
 
@@ -342,9 +349,8 @@ gethostnm(ia)
  * Search cache.
  */
 	for (i = 0; i < hcx; i++) {
-		if (u[3] != hc[i].addr[3] || u[2] != hc[i].addr[2]
-		||  u[1] != hc[i].addr[1] || u[0] != hc[i].addr[0])
-			continue;
+	    if (u[3] == hc[i].addr[3] && u[2] == hc[i].addr[2]
+	    &&  u[1] == hc[i].addr[1] && u[0] == hc[i].addr[0])
 		return(hc[i].name);
 	}
 /*
@@ -354,37 +360,34 @@ gethostnm(ia)
  * address.
  */
 	if (Fhost)
-		he = gethostbyaddr((char *)ia, sizeof(struct in_addr), AF_INET);
+	    he = gethostbyaddr((char *)ia, sizeof(struct in_addr), AF_INET);
 	if (!he || !he->h_name) {
-		(void) sprintf(hbuf, "%u.%u.%u.%u", u[0], u[1], u[2], u[3]);
-		hn = hbuf;
+	    (void) sprintf(hbuf, "%u.%u.%u.%u", u[0], u[1], u[2], u[3]);
+	    hn = hbuf;
 	} else
-		hn = (char *)he->h_name;
+	    hn = (char *)he->h_name;
 /*
  * Allocate space for name and copy name to it.
  */
-	if (!(np = (char *)malloc((MALLOC_S)(strlen(hn) + 1)))) {
-		(void) fprintf(stderr, "%s: no space for host name: %s\n",
-			Pn, hn);
-		Exit(1);
+	if (!(np = mkstrcpy(hn, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no space for host name: ", Pn);
+	    safestrprt(hn, stderr, 1);
+	    Exit(1);
 	}
-	(void) strcpy(np, hn);
 /*
  * Add address/name entry to cache.  Allocate cache space in HCINC chunks.
  */
 	if (hcx >= nhc) {
-		nhc += HCINC;
-		if (!hc)
-			hc = (struct hostcache *)malloc(
-				(MALLOC_S)(sizeof(struct hostcache) * nhc));
-		else
-			hc = (struct hostcache *)realloc((MALLOC_P *)hc,
-				(MALLOC_S)(sizeof(struct hostcache) * nhc));
-		if (!hc) {
-			(void) fprintf(stderr, "%s: no space for host cache\n",
-				Pn);
-			Exit(1);
-		}
+	    nhc += HCINC;
+	    len = (MALLOC_S)(nhc * sizeof(struct hostcache));
+	    if (!hc)
+		hc = (struct hostcache *)malloc(len);
+	    else
+		hc = (struct hostcache *)realloc((MALLOC_P *)hc, len);
+	    if (!hc) {
+		(void) fprintf(stderr, "%s: no space for host cache\n", Pn);
+		Exit(1);
+	    }
 	}
 	hc[hcx].addr[0] = u[0];
 	hc[hcx].addr[1] = u[1];
@@ -493,14 +496,12 @@ lkup_port(p, pr, src)
  *
  * Return a pointer to the name.
  */
-	nl = strlen(pn);
-	if (!(nm = malloc(nl + 1))) {
+	if (!(nm = mkstrcpy(pn, &nl))) {
 	    (void) fprintf(stderr,
-		"%s: can't allocate %d bytes for port name: %s\n",
-		Pn, (int)(nl + 1), pn);
+		"%s: can't allocate space for port name: ", Pn);
+	    safestrprt(pn, stderr, 1);
 	    Exit(1);
 	}
-	(void) strcpy(nm, pn);
 	pt->name = nm;
 	pt->nl = nl;
 	pt->port = p;
@@ -622,15 +623,14 @@ print_file()
 /*
  * Size or print the command.
  */
+	cp = (Lp->cmd && *Lp->cmd != '\0') ? Lp->cmd : "(unknown)";
 	if (!PrPass) {
-	    cp = Lp->cmd ? Lp->cmd : "(unknown)";
-	    if ((len = strlen(cp)) > CMDL)
+	    if ((len = safestrlen(cp)) > CMDL)
 		len = CMDL;
 	    if (len > CmdColW)
 		CmdColW = len;
 	} else
-	    (void) printf("%-*.*s", CmdColW, CmdColW,
-		Lp->cmd ? Lp->cmd : "(unknown)");
+	    safestrprtn(cp, CmdColW, stdout, 0);
 /*
  * Size or print the process ID.
  */
@@ -951,7 +951,7 @@ addr_too_long:
 	    }
 	}
 	if (Namech[0]) {
-	    (void) fputs(Namech, stdout);
+	    safestrprt(Namech, stdout, 0);
 	    return(1);
 	}
 	return(0);
@@ -1001,108 +1001,108 @@ printiproto(p)
 
 #if	defined(IPPROTO_TCP)
 	case IPPROTO_TCP:
-		s = "TCP";
-		break;
-#endif
+	    s = "TCP";
+	    break;
+#endif	/* defined(IPPROTO_TCP) */
 
 #if	defined(IPPROTO_UDP)
 	case IPPROTO_UDP:
-		s = "UDP";
-		break;
-#endif
+	    s = "UDP";
+	    break;
+#endif	/* defined(IPPROTO_UDP) */
 
 #if	defined(IPPROTO_IP)
 	case IPPROTO_IP:
-		s = "IP";
-		break;
-#endif
+	    s = "IP";
+	    break;
+#endif	/* defined(IPPROTO_IP) */
 
 #if	defined(IPPROTO_ICMP)
 	case IPPROTO_ICMP:
-		s = "ICMP";
-		break;
-#endif
+	    s = "ICMP";
+	    break;
+#endif	/* defined(IPPROTO_ICMP) */
 
 #if	defined(IPPROTO_IGMP)
 	case IPPROTO_IGMP:
-		s = "IGMP";
-		break;
-#endif
+	    s = "IGMP";
+	    break;
+#endif	/* defined(IPPROTO_IGMP) */
 
 #if	defined(IPPROTO_GGP)
 	case IPPROTO_GGP:
-		s = "GGP";
-		break;
-#endif
+	    s = "GGP";
+	    break;
+#endif	/* defined(IPPROTO_GGP) */
 
 #if	defined(IPPROTO_EGP)
 	case IPPROTO_EGP:
-		s = "EGP";
-		break;
-#endif
+	    s = "EGP";
+	    break;
+#endif	/* defined(IPPROTO_EGP) */
 
 #if	defined(IPPROTO_PUP)
 	case IPPROTO_PUP:
-		s = "PUP";
-		break;
-#endif
+	    s = "PUP";
+	    break;
+#endif	/* defined(IPPROTO_PUP) */
 
 #if	defined(IPPROTO_IDP)
 	case IPPROTO_IDP:
-		s = "IDP";
-		break;
-#endif
+	    s = "IDP";
+	    break;
+#endif	/* defined(IPPROTO_IDP) */
 
 #if	defined(IPPROTO_ND)
 	case IPPROTO_ND:
-		s = "ND";
-		break;
-#endif
+	    s = "ND";
+	    break;
+#endif	/* defined(IPPROTO_ND) */
 
 #if	defined(IPPROTO_RAW)
 	case IPPROTO_RAW:
-		s = "RAW";
-		break;
-#endif
+	    s = "RAW";
+	    break;
+#endif	/* defined(IPPROTO_RAW) */
 
 #if	defined(IPPROTO_HELLO)
 	case IPPROTO_HELLO:
-		s = "HELLO";
-		break;
-#endif
+	    s = "HELLO";
+	    break;
+#endif	/* defined(IPPROTO_HELLO) */
 
 #if	defined(IPPROTO_PXP)
 	case IPPROTO_PXP:
-		s = "PXP";
-		break;
-#endif
+	    s = "PXP";
+	    break;
+#endif	/* defined(IPPROTO_PXP) */
 
 #if	defined(IPPROTO_RAWIP)
 	case IPPROTO_RAWIP:
-		s = "RAWIP";
-		break;
-#endif
+	    s = "RAWIP";
+	    break;
+#endif	/* defined(IPPROTO_RAWIP) */
 
 #if	defined(IPPROTO_RAWIF)
 	case IPPROTO_RAWIF:
-		s = "RAWIF";
-		break;
-#endif
+	    s = "RAWIF";
+	    break;
+#endif	/* defined(IPPROTO_RAWIF) */
 
 	default:
-		s = (char *)NULL;
+	    s = (char *)NULL;
 	}
 	if (s)
-		(void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, s);
+	    (void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, s);
 	else {	
-		if (m < 0) {
-			for (i = 0, m = 1; i < IPROTOL-2; i++)
-				m *= 10;
-		}
-		if (m > p)
-			(void) sprintf(Lf->iproto, "%d?", p);
-		else
-			(void) sprintf(Lf->iproto, "*%d?", p % (m/10));
+	    if (m < 0) {
+		for (i = 0, m = 1; i < IPROTOL-2; i++)
+		    m *= 10;
+	    }
+	    if (m > p)
+		(void) sprintf(Lf->iproto, "%d?", p);
+	    else
+		(void) sprintf(Lf->iproto, "*%d?", p % (m/10));
 	}
 }
 
@@ -1129,7 +1129,7 @@ printname(nl)
 	/*
 	 * Print the name characters, if there are some.
 	 */
-	    (void) fputs(Lf->nm, stdout);
+	    safestrprt(Lf->nm, stdout, 0);
 	    ps++;
 	    if (!Lf->li[0].as && !Lf->li[1].as)
 		goto print_nma;
@@ -1186,7 +1186,7 @@ printname(nl)
 	 * possible path name components.
 	 */
 	    if (Lf->fsdir) {
-		(void) fputs(Lf->fsdir, stdout);
+		safestrprt(Lf->fsdir, stdout, 0);
 		ps++;
 	    }
 
@@ -1203,7 +1203,7 @@ printname(nl)
 		    }
 		} else
 		    (void) fputs(" -- ", stdout);
-		(void) fputs(cp, stdout);
+		safestrprt(cp, stdout, 0);
 		ps++;
 		goto print_nma;
 	    }
@@ -1214,7 +1214,7 @@ printname(nl)
 		    (void) fputs(" (", stdout);
 		else
 		    (void) putchar('(');
-		(void) fputs(Lf->fsdev, stdout);
+		safestrprt(Lf->fsdev, stdout, 0);
 		(void) putchar(')');
 		ps++;
 	    }
@@ -1229,7 +1229,7 @@ print_nma:
 	if (Lf->nma) {
 	    if (ps)
 		putchar(' ');
-	    (void) fputs(Lf->nma, stdout);
+	    safestrprt(Lf->nma, stdout, 0);
 	    ps++;
 	}
 /*
@@ -1266,21 +1266,21 @@ printrawaddr(sa)
 	struct sockaddr *sa;		/* socket address */
 {
 	(void) sprintf(endnm(), "%u/%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u",
-		sa->sa_family,
-		(unsigned char)sa->sa_data[0],
-		(unsigned char)sa->sa_data[1],
-		(unsigned char)sa->sa_data[2],
-		(unsigned char)sa->sa_data[3],
-		(unsigned char)sa->sa_data[4],
-		(unsigned char)sa->sa_data[5],
-		(unsigned char)sa->sa_data[6],
-		(unsigned char)sa->sa_data[7],
-		(unsigned char)sa->sa_data[8],
-		(unsigned char)sa->sa_data[9],
-		(unsigned char)sa->sa_data[10],
-		(unsigned char)sa->sa_data[11],
-		(unsigned char)sa->sa_data[12],
-		(unsigned char)sa->sa_data[13]);
+	    sa->sa_family,
+	    (unsigned char)sa->sa_data[0],
+	    (unsigned char)sa->sa_data[1],
+	    (unsigned char)sa->sa_data[2],
+	    (unsigned char)sa->sa_data[3],
+	    (unsigned char)sa->sa_data[4],
+	    (unsigned char)sa->sa_data[5],
+	    (unsigned char)sa->sa_data[6],
+	    (unsigned char)sa->sa_data[7],
+	    (unsigned char)sa->sa_data[8],
+	    (unsigned char)sa->sa_data[9],
+	    (unsigned char)sa->sa_data[10],
+	    (unsigned char)sa->sa_data[11],
+	    (unsigned char)sa->sa_data[12],
+	    (unsigned char)sa->sa_data[13]);
 }
 
 
@@ -1425,289 +1425,288 @@ printunkaf(fam)
 
 #if	defined(AF_UNSPEC)
 	case AF_UNSPEC:
-		s = "UNSPEC";
-		break;
+	    s = "UNSPEC";
+	    break;
 #endif	/* defined(AF_UNSPEC) */
 
 #if	defined(AF_UNIX)
 	case AF_UNIX:
-		s = "UNIX";
-		break;
+	    s = "UNIX";
+	    break;
 #endif	/* defined(AF_UNIX) */
 
 #if	defined(AF_INET)
 	case AF_INET:
-		s = "INET";
-		break;
+	    s = "INET";
+	    break;
 #endif	/* defined(AF_INET) */
 
 #if	defined(AF_IMPLINK)
 	case AF_IMPLINK:
-		s = "IMPLINK";
-		break;
+	    s = "IMPLINK";
+	    break;
 #endif	/* defined(AF_IMPLINK) */
 
 #if	defined(AF_PUP)
 	case AF_PUP:
-		s = "PUP";
-		break;
+	    s = "PUP";
+	    break;
 #endif	/* defined(AF_PUP) */
 
 #if	defined(AF_CHAOS)
 	case AF_CHAOS:
-		s = "CHAOS";
-		break;
+	    s = "CHAOS";
+	    break;
 #endif	/* defined(AF_CHAOS) */
 
 #if	defined(AF_NS)
 	case AF_NS:
-		s = "NS";
-		break;
+	    s = "NS";
+	    break;
 #endif	/* defined(AF_NS) */
 
 #if	defined(AF_ISO)
 	case AF_ISO:
-		s = "ISO";
-		break;
+	    s = "ISO";
+	    break;
 #endif	/* defined(AF_ISO) */
 
 #if	defined(AF_NBS)
 # if	!defined(AF_ISO) || AF_NBS!=AF_ISO
 	case AF_NBS:
-		s = "NBS";
-		break;
+	    s = "NBS";
+	    break;
 # endif	/* !defined(AF_ISO) || AF_NBS!=AF_ISO */
 #endif	/* defined(AF_NBS) */
 
 #if	defined(AF_ECMA)
 	case AF_ECMA:
-		s = "ECMA";
-		break;
+	    s = "ECMA";
+	    break;
 #endif	/* defined(AF_ECMA) */
 
 #if	defined(AF_DATAKIT)
 	case AF_DATAKIT:
-		s = "DATAKIT";
-		break;
+	    s = "DATAKIT";
+	    break;
 #endif	/* defined(AF_DATAKIT) */
 
 #if	defined(AF_CCITT)
 	case AF_CCITT:
-		s = "CCITT";
-		break;
+	    s = "CCITT";
+	    break;
 #endif	/* defined(AF_CCITT) */
 
 #if	defined(AF_SNA)
 	case AF_SNA:
-		s = "SNA";
-		break;
+	    s = "SNA";
+	    break;
 #endif	/* defined(AF_SNA) */
 
 #if	defined(AF_DECnet)
 	case AF_DECnet:
-		s = "DECnet";
-		break;
+	    s = "DECnet";
+	    break;
 #endif	/* defined(AF_DECnet) */
 
 #if	defined(AF_DLI)
 	case AF_DLI:
-		s = "DLI";
-		break;
+	    s = "DLI";
+	    break;
 #endif	/* defined(AF_DLI) */
 
 #if	defined(AF_LAT)
 	case AF_LAT:
-		s = "LAT";
-		break;
+	    s = "LAT";
+	    break;
 #endif	/* defined(AF_LAT) */
 
 #if	defined(AF_HYLINK)
 	case AF_HYLINK:
-		s = "HYLINK";
-		break;
+	    s = "HYLINK";
+	    break;
 #endif	/* defined(AF_HYLINK) */
 
 #if	defined(AF_APPLETALK)
 	case AF_APPLETALK:
-		s = "APPLETALK";
-		break;
+	    s = "APPLETALK";
+	    break;
 #endif	/* defined(AF_APPLETALK) */
 
 #if	defined(AF_BSC)
 	case AF_BSC:
-		s = "BSC";
-		break;
+	    s = "BSC";
+	    break;
 #endif	/* defined(AF_BSC) */
 
 #if	defined(AF_DSS)
 	case AF_DSS:
-		s = "DSS";
-		break;
+	    s = "DSS";
+	    break;
 #endif	/* defined(AF_DSS) */
 
 #if	defined(AF_ROUTE)
 	case AF_ROUTE:
-		s = "ROUTE";
-		break;
+	    s = "ROUTE";
+	    break;
 #endif	/* defined(AF_ROUTE) */
 
 #if	defined(AF_RAW)
 	case AF_RAW:
-		s = "RAW";
-		break;
+	    s = "RAW";
+	    break;
 #endif	/* defined(AF_RAW) */
 
 #if	defined(AF_LINK)
 	case AF_LINK:
-		s = "LINK";
-		break;
+	    s = "LINK";
+	    break;
 #endif	/* defined(AF_LINK) */
 
 #if	defined(pseudo_AF_XTP)
 	case pseudo_AF_XTP:
-		p = "pseudo_";
-		s = "XTP";
-		break;
+	    p = "pseudo_";
+	    s = "XTP";
+	    break;
 #endif	/* defined(pseudo_AF_XTP) */
 
 #if	defined(AF_RMP)
 	case AF_RMP:
-		s = "RMP";
-		break;
+	    s = "RMP";
+	    break;
 #endif	/* defined(AF_RMP) */
 
 #if	defined(AF_COIP)
 	case AF_COIP:
-		s = "COIP";
-		break;
+	    s = "COIP";
+	    break;
 #endif	/* defined(AF_COIP) */
 
 #if	defined(AF_CNT)
 	case AF_CNT:
-		s = "CNT";
-		break;
+	    s = "CNT";
+	    break;
 #endif	/* defined(AF_CNT) */
 
 #if	defined(pseudo_AF_RTIP)
 	case pseudo_AF_RTIP:
-		p = "pseudo_";
-		s = "RTIP";
-		break;
+	    p = "pseudo_";
+	    s = "RTIP";
+	    break;
 #endif	/* defined(pseudo_AF_RTIP) */
 
 #if	defined(AF_NETMAN)
 	case AF_NETMAN:
-		s = "NETMAN";
-		break;
+	    s = "NETMAN";
+	    break;
 #endif	/* defined(AF_NETMAN) */
 
 #if	defined(AF_INTF)
 	case AF_INTF:
-		s = "INTF";
-		break;
+	    s = "INTF";
+	    break;
 #endif	/* defined(AF_INTF) */
 
 #if	defined(AF_NETWARE)
 	case AF_NETWARE:
-		s = "NETWARE";
-		break;
+	    s = "NETWARE";
+	    break;
 #endif	/* defined(AF_NETWARE) */
 
 #if	defined(AF_NDD)
 	case AF_NDD:
-		s = "NDD";
-		break;
+	    s = "NDD";
+	    break;
 #endif	/* defined(AF_NDD) */
 
 #if	defined(AF_NIT)
 # if	!defined(AF_ROUTE) || AF_ROUTE!=AF_NIT
 	case AF_NIT:
-		s = "NIT";
-		break;
+	    s = "NIT";
+	    break;
 # endif	/* !defined(AF_ROUTE) || AF_ROUTE!=AF_NIT */
 #endif	/* defined(AF_NIT) */
 
 #if	defined(AF_802)
 # if	!defined(AF_RAW) || AF_RAW!=AF_802
 	case AF_802:
-		s = "802";
-		break;
+	    s = "802";
+	    break;
 # endif	/* !defined(AF_RAW) || AF_RAW!=AF_802 */
 #endif	/* defined(AF_802) */
 
 #if	defined(AF_X25)
 	case AF_X25:
-		s = "X25";
-		break;
+	    s = "X25";
+	    break;
 #endif	/* defined(AF_X25) */
 
 #if	defined(AF_CTF)
 	case AF_CTF:
-		s = "CTF";
-		break;
+	    s = "CTF";
+	    break;
 #endif	/* defined(AF_CTF) */
 
 #if	defined(AF_WAN)
 	case AF_WAN:
-		s = "WAN";
-		break;
+	    s = "WAN";
+	    break;
 #endif	/* defined(AF_WAN) */
 
 #if	defined(AF_OSINET)
 # if	defined(AF_INET) && AF_INET!=AF_OSINET
 	case AF_OSINET:
-		s = "OSINET";
-		break;
+	    s = "OSINET";
+	    break;
 # endif	/* defined(AF_INET) && AF_INET!=AF_OSINET */
 #endif	/* defined(AF_OSINET) */
 
 #if	defined(AF_GOSIP)
 	case AF_GOSIP:
-		s = "GOSIP";
-		break;
+	    s = "GOSIP";
+	    break;
 #endif	/* defined(AF_GOSIP) */
 
 #if	defined(AF_SDL)
 	case AF_SDL:
-		s = "SDL";
-		break;
+	    s = "SDL";
+	    break;
 #endif	/* defined(AF_SDL) */
 
 #if	defined(AF_IPX)
 	case AF_IPX:
-		s = "IPX";
-		break;
+	    s = "IPX";
+	    break;
 #endif	/* defined(AF_IPX) */
 
 #if	defined(AF_SIP)
 	case AF_SIP:
-		s = "SIP";
-		break;
+	    s = "SIP";
+	    break;
 #endif	/* defined(AF_SIP) */
 
 #if	defined(psuedo_AF_PIP)
 	case psuedo_AF_PIP:
-		p = "pseudo_";
-		s = "PIP";
-		break;
+	    p = "pseudo_";
+	    s = "PIP";
+	    break;
 #endif	/* defined(psuedo_AF_PIP) */
 
 #if	defined(AF_OTS)
 	case AF_OTS:
-		s = "OTS";
-		break;
+	    s = "OTS";
+	    break;
 #endif	/* defined(AF_OTS) */
 
 #if	defined(AF_USER)
 	case AF_USER:
-		s = "USER";
-		break;
+	    s = "USER";
+	    break;
 #endif	/* defined(AF_USER) */
 
 	default:
-		(void) sprintf(Namech, "no further information on family %#x",
-			fam);
-		return;
+	    (void) sprintf(Namech, "no further information on family %#x", fam);
+	    return;
 	}
 	(void) sprintf(Namech, "no further information on %sAF_%s", p, s);
 	return;
