@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.5 97/10/10 08:23:37 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.10 98/03/06 07:31:02 abe Exp $";
 #endif
 
 
@@ -51,9 +51,9 @@ comppid(a1, a2)
 	struct lproc **p2 = (struct lproc **)a2;
 
 	if ((*p1)->pid < (*p2)->pid)
-		return(-1);
+	    return(-1);
 	if ((*p1)->pid > (*p2)->pid)
-		return(1);
+	    return(1);
 	return(0);
 }
 
@@ -79,6 +79,8 @@ alloc_lfile(nm, num)
 		(void) free((FREE_P *)Lf->dev_ch);
 	    if (Lf->nm)
 		(void) free((FREE_P *)Lf->nm);
+	    if (Lf->nma)
+		(void) free((FREE_P *)Lf->nma);
 /*
  * Othwerise, allocate a new structure.
  */
@@ -129,7 +131,7 @@ alloc_lfile(nm, num)
 		(void) sprintf(Lf->fd, "*%03d", num % 1000);
 	} else
 	    Lf->fd[0] = '\0';
-	Lf->dev_ch = Lf->fsdir = Lf->fsdev = Lf->nm = Lf->nma = NULL;
+	Lf->dev_ch = Lf->fsdir = Lf->fsdev = Lf->nm = Lf->nma = (char *)NULL;
 	Lf->ch = -1;
 
 #if	defined(HASNCACHE)
@@ -150,9 +152,9 @@ alloc_lfile(nm, num)
 /*
  * See if the file descriptor has been selected.
  */
-	if (Fdl == NULL || (nm == NULL && num < 0))
+	if (!Fdl || (!nm && num < 0))
 	    return;
-	if ((cp = nm) != NULL) {
+	if ((cp = nm)) {
 	    while (*cp && *cp == ' ')
 		cp++;
 	} else {
@@ -182,13 +184,14 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
 	int pss;			/* process select state */
 	int sf;				/* process select flags */
 {
+	char *cp, *dp;
 	MALLOC_S len;
-	static int sz;
+	static int sz = 0;
 
-	if (Lproc == NULL) {
-	    if ((Lproc = (struct lproc *)malloc(
-			 (MALLOC_S)(LPROCINCR * sizeof(struct lproc))))
-	    == NULL) {
+	if (!Lproc) {
+	    if (!(Lproc = (struct lproc *)malloc(
+			  (MALLOC_S)(LPROCINCR * sizeof(struct lproc)))))
+	    {
 		(void) fprintf(stderr,
 		    "%s: no malloc space for %d local proc structures\n",
 		    Pn, LPROCINCR);
@@ -197,9 +200,9 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
 	    sz = LPROCINCR;
 	} else if ((Nlproc + 1) > sz) {
 	    sz += LPROCINCR;
-	    if ((Lproc = (struct lproc *)realloc((MALLOC_P *)Lproc,
-			 (MALLOC_S)(sz * sizeof(struct lproc))))
-	    == NULL) {
+	    if (!(Lproc = (struct lproc *)realloc((MALLOC_P *)Lproc,
+			  (MALLOC_S)(sz * sizeof(struct lproc)))))
+	    {
 		(void) fprintf(stderr,
 		    "%s: no realloc space for %d local proc structures\n",
 		    Pn, sz);
@@ -210,25 +213,19 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
 	Lp->pid = pid;
 	Lp->pgrp = pgrp;
 	Lp->ppid = ppid;
-	Lp->file = NULL;
+	Lp->file = (struct lfile *)NULL;
 	Lp->sf = (short)sf;
 	Lp->pss = (short)pss;
 	Lp->uid = (uid_t)uid;
 /*
  * Allocate space for the full command name and copy it there.
- * Note: only field output can use more than CMDL-1 command name characters.
  */
-	if ((len = strlen(cmd)) == 0) {
-	    Lp->cmd = (char *)NULL;
-	    return;
-	}
-	if ((Lp->cmd = (char *)malloc(len + 1)) == (char *)NULL) {
-	    (void) fprintf(stderr,
-		"%s: no space for PID %d command name: %s\n",
-		Pn, pid, cmd);
+	if (!(Lp->cmd = mkstrcpy(cmd, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: PID %d, no space for command name: ",
+		Pn, pid);
+	    safestrprt(cmd, stderr, 1);
 	    Exit(1);
 	}
-	(void) strcpy(Lp->cmd, cmd);
 }
 
 
@@ -295,7 +292,11 @@ examine_lproc()
 	    }
 	}
 	if (Lp->pss && Npid == 1 && sbp) {
+	    print_init();
 	    print_proc();
+	    PrPass++;
+	    if (PrPass < 2)
+		print_proc();
 	    Lp->pss = 0;
 	}
 /*
@@ -564,7 +565,7 @@ link_lfile()
 		Fnet = 2;
 	if (Fnfs && (Lf->sf & SELNFS))
 		Fnfs = 2;
-	Lf = NULL;
+	Lf = (struct lfile *)NULL;
 }
 
 
@@ -590,206 +591,191 @@ print_proc()
 	 * selected.  If options are being OR'd, print the PID;
 	 * if AND'd, see if anything has been selected.
 	 */
-		if (Fand) {
-			for (Lf = Lp->file; Lf; Lf = Lf->next) {
-				if (is_file_sel(Lf)) {
-					sel++;
-					break;
-				}
-			}
-		} else
-			sel = 1;
-		if (sel)
-			(void) printf("%d\n", Lp->pid);
-		return;
+	    if (Fand) {
+		for (Lf = Lp->file; Lf; Lf = Lf->next) {
+		    if (is_file_sel(Lf)) {
+			sel++;
+			break;
+		    }
+		}
+	    } else
+		sel = 1;
+	    if (sel)
+		(void) printf("%d\n", Lp->pid);
+	    return;
 	}
 /*
  * If fields have been selected, output the process-only ones, provided
  * that some file has also been selected.
  */
 	if (Ffield) {
-		for (Lf = Lp->file; Lf; Lf = Lf->next) {
-			if (is_file_sel(Lf))
-				break;
-		}
-		if (Lf == (struct lfile *)NULL)
-			return;
-		(void) printf("%c%d%c",
-			LSOF_FID_PID, Lp->pid, Terminator);
-		if (FieldSel[LSOF_FIX_PGRP].st)
-			(void) printf("%c%d%c",
-				LSOF_FID_PGRP, Lp->pgrp, Terminator);
+	    for (Lf = Lp->file; Lf; Lf = Lf->next) {
+		if (is_file_sel(Lf))
+		    break;
+	    }
+	    if (!Lf)
+		return;
+	    (void) printf("%c%d%c", LSOF_FID_PID, Lp->pid, Terminator);
+	    if (FieldSel[LSOF_FIX_PGRP].st && Fpgrp)
+		(void) printf("%c%d%c", LSOF_FID_PGRP, Lp->pgrp, Terminator);
 
 #if	defined(HASPPID)
-		if (FieldSel[LSOF_FIX_PPID].st)
-			(void) printf("%c%d%c",
-				LSOF_FID_PPID, Lp->ppid, Terminator);
-#endif	/* defined(HSPPID) */
+	    if (FieldSel[LSOF_FIX_PPID].st && Fppid)
+		(void) printf("%c%d%c", LSOF_FID_PPID, Lp->ppid, Terminator);
+#endif	/* defined(HASPPID) */
 
-		if (FieldSel[LSOF_FIX_CMD].st)
-			(void) printf("%c%s%c",
-				LSOF_FID_CMD,
-				Lp->cmd ? Lp->cmd : "(unknown)",
-				Terminator);
-		if (FieldSel[LSOF_FIX_UID].st)
-			(void) printf("%c%d%c",
-				LSOF_FID_UID, Lp->uid, Terminator);
-		if (FieldSel[LSOF_FIX_LOGIN].st) {
-			cp = printuid((UID_ARG)Lp->uid, &ty);
-			if (ty == 0)
-				(void) printf("%c%s%c",
-					LSOF_FID_LOGIN, cp, Terminator);
-		}
-		if (Terminator == '\0')
-			putchar('\n');
+	    if (FieldSel[LSOF_FIX_CMD].st)
+		(void) printf("%c%s%c", LSOF_FID_CMD,
+		    Lp->cmd ? Lp->cmd : "(unknown)",
+		    Terminator);
+	    if (FieldSel[LSOF_FIX_UID].st)
+		(void) printf("%c%d%c", LSOF_FID_UID, (int)Lp->uid, Terminator);
+	    if (FieldSel[LSOF_FIX_LOGIN].st) {
+		cp = printuid((UID_ARG)Lp->uid, &ty);
+		if (ty == 0)
+		    (void) printf("%c%s%c", LSOF_FID_LOGIN, cp, Terminator);
+	    }
+	    if (Terminator == '\0')
+	    putchar('\n');
 	}
 /*
  * Print files.
  */
 	for (Lf = Lp->file; Lf; Lf = Lf->next) {
-		if (!is_file_sel(Lf))
-			continue;
+	    if (!is_file_sel(Lf))
+		continue;
 	/*
 	 * If no field output selected, print dialects-specific formatted
 	 * output.
 	 */
-		if (!Ffield) {
-			print_file();
-			continue;
-		}
+	    if (!Ffield) {
+		print_file();
+		continue;
+	    }
 	/*
 	 * Print selected fields.
 	 */
-		lc = st = 0;
-		if (FieldSel[LSOF_FIX_FD].st) {
-			for (cp = Lf->fd; *cp == ' '; cp++)
-				;
-			if (*cp) {
-				(void) printf("%c%s%c",
-					LSOF_FID_FD, cp, Terminator);
-				lc++;
-			}
+	    lc = st = 0;
+	    if (FieldSel[LSOF_FIX_FD].st) {
+		for (cp = Lf->fd; *cp == ' '; cp++)
+		    ;
+		if (*cp) {
+		    (void) printf("%c%s%c", LSOF_FID_FD, cp, Terminator);
+		    lc++;
 		}
-		if (FieldSel[LSOF_FIX_ACCESS].st) {
-			(void) printf("%c%c%c",
-				LSOF_FID_ACCESS, Lf->access, Terminator);
-			lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_ACCESS].st) {
+		(void) printf("%c%c%c",
+		    LSOF_FID_ACCESS, Lf->access, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_LOCK].st) {
+		(void) printf("%c%c%c", LSOF_FID_LOCK, Lf->lock, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_TYPE].st) {
+		for (cp = Lf->type; *cp == ' '; cp++)
+		    ;
+		if (*cp) {
+		    (void) printf("%c%s%c", LSOF_FID_TYPE, cp, Terminator);
+		    lc++;
 		}
-		if (FieldSel[LSOF_FIX_LOCK].st) {
-			(void) printf("%c%c%c",
-				LSOF_FID_LOCK, Lf->lock, Terminator);
-			lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_DEVCH].st && Lf->dev_ch && Lf->dev_ch[0]) {
+		for (cp = Lf->dev_ch; *cp == ' '; cp++)
+		    ;
+		if (*cp) {
+		    (void) printf("%c%s%c", LSOF_FID_DEVCH, cp, Terminator);
+		    lc++;
 		}
-		if (FieldSel[LSOF_FIX_TYPE].st) {
-			for (cp = Lf->type; *cp == ' '; cp++)
-				;
-			if (*cp) {
-				(void) printf("%c%s%c",
-					LSOF_FID_TYPE, cp, Terminator);
-				lc++;
-			}
+	    }
+	    if (FieldSel[LSOF_FIX_DEVN].st && Lf->dev_def) {
+		(void) printf("%c0x%lx%c", LSOF_FID_DEVN,
+		    (unsigned long)Lf->dev, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_SIZE].st && Lf->sz_def) {
+		putchar(LSOF_FID_SIZE);
+		(void) printf(SzOffFmt_d, Lf->sz);
+		putchar(Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_OFFSET].st && Lf->off_def) {
+		putchar(LSOF_FID_OFFSET);
+		(void) printf(SzOffFmt_d, Lf->off);
+		putchar(Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_INODE].st && Lf->inp_ty == 1) {
+		(void) printf("%c%lu%c", LSOF_FID_INODE, Lf->inode, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_PROTO].st && Lf->inp_ty == 2) {
+		for (cp = Lf->iproto; *cp == ' '; cp++)
+		    ;
+		if (*cp) {
+		    (void) printf("%c%s%c", LSOF_FID_PROTO, cp, Terminator);
+		    lc++;
 		}
-		if (FieldSel[LSOF_FIX_DEVCH].st && Lf->dev_ch && Lf->dev_ch[0]) {
-			for (cp = Lf->dev_ch; *cp == ' '; cp++)
-				;
-			if (*cp) {
-				(void) printf("%c%s%c",
-					LSOF_FID_DEVCH, cp, Terminator);
-				lc++;
-			}
-		}
-		if (FieldSel[LSOF_FIX_DEVN].st && Lf->dev_def) {
-			(void) printf("%c0x%lx%c",
-				LSOF_FID_DEVN, (unsigned long)Lf->dev,
-				Terminator);
-			lc++;
-		}
-		if (FieldSel[LSOF_FIX_SIZE].st && Lf->sz_def) {
-			(void) printf("%c%d%c",
-				LSOF_FID_SIZE, Lf->sz, Terminator);
-			lc++;
-		}
-		if (FieldSel[LSOF_FIX_OFFSET].st && Lf->off_def) {
-			if ((unsigned int)Lf->off < 100000000)
-				(void) printf("%c0t%d%c",
-					LSOF_FID_OFFSET, Lf->off, Terminator);
-			else
-				(void) printf("%c0x%x%c",
-					LSOF_FID_OFFSET, Lf->off, Terminator);
-			lc++;
-		}
-		if (FieldSel[LSOF_FIX_INODE].st && Lf->inp_ty == 1) {
-			(void) printf("%c%lu%c",
-				LSOF_FID_INODE, Lf->inode, Terminator);
-			lc++;
-		}
-		if (FieldSel[LSOF_FIX_PROTO].st && Lf->inp_ty == 2) {
-			for (cp = Lf->iproto; *cp == ' '; cp++)
-				;
-			if (*cp) {
-				(void) printf("%c%s%c",
-					LSOF_FID_PROTO, cp, Terminator);
-				lc++;
-			}
-		}
-		if (FieldSel[LSOF_FIX_STREAM].st && Lf->nm && Lf->is_stream) {
-			if (strncmp(Lf->nm, "STR:", 4) == 0
-			||  strcmp(Lf->iproto, "STR") == 0) {
-				putchar(FieldSel[LSOF_FIX_STREAM].id);
-				printname(0);
-				putchar(Terminator);
-				lc++;
-				st++;
-			}
-		}
-		if (st == 0 && FieldSel[LSOF_FIX_NAME].st) {
-			putchar(FieldSel[LSOF_FIX_NAME].id);
-			printname(0);
-			putchar(Terminator);
-			lc++;
-		}
-		if (Lf->lts.type >= 0 && FieldSel[LSOF_FIX_TCPTPI].st) {
-			print_tcptpi(0);
-			lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_STREAM].st && Lf->nm && Lf->is_stream) {
+		if (strncmp(Lf->nm, "STR:", 4) == 0
+		||  strcmp(Lf->iproto, "STR") == 0) {
+		    putchar(FieldSel[LSOF_FIX_STREAM].id);
+		    printname(0);
+		    putchar(Terminator);
+		    lc++;
+		    st++;
 		}
+	    }
+	    if (st == 0 && FieldSel[LSOF_FIX_NAME].st) {
+		putchar(FieldSel[LSOF_FIX_NAME].id);
+		printname(0);
+		putchar(Terminator);
+		lc++;
+	    }
+	    if (Lf->lts.type >= 0 && FieldSel[LSOF_FIX_TCPTPI].st) {
+		print_tcptpi(0);
+		lc++;
+	    }
 
 #if	defined(HASFIELDAP1)
-		LISTLFILEAP1
+	    LISTLFILEAP1
 #endif	/* defined(HASFIELDAP1) */
 
 #if	defined(HASFIELDAP2)
-		LISTLFILEAP2
+	    LISTLFILEAP2
 #endif	/* defined(HASFIELDAP2) */
 
 #if	defined(HASFIELDAP3)
-		LISTLFILEAP3
+	    LISTLFILEAP3
 #endif	/* defined(HASFIELDAP3) */
 
 #if	defined(HASFIELDAP4)
-		LISTLFILEAP4
+	    LISTLFILEAP4
 #endif	/* defined(HASFIELDAP4) */
 
 #if	defined(HASFIELDAP5)
-		LISTLFILEAP5
+	    LISTLFILEAP5
 #endif	/* defined(HASFIELDAP5) */
 
 #if	defined(HASFIELDAP6)
-		LISTLFILEAP6
+	    LISTLFILEAP6
 #endif	/* defined(HASFIELDAP6) */
 
 #if	defined(HASFIELDAP7)
-		LISTLFILEAP7
+	    LISTLFILEAP7
 #endif	/* defined(HASFIELDAP7) */
 
 #if	defined(HASFIELDAP8)
-		LISTLFILEAP8
+	    LISTLFILEAP8
 #endif	/* defined(HASFIELDAP8) */
 
 #if	defined(HASFIELDAP9)
-		LISTLFILEAP9
+	    LISTLFILEAP9
 #endif	/* defined(HASFIELDAP9) */
 
-		if (Terminator == '\0' && lc)
-			putchar('\n');
+	    if (Terminator == '\0' && lc)
+		putchar('\n');
 	}
 }
